@@ -1,4 +1,4 @@
-/* $Id: line.c,v 1.1.1.1 2001/05/12 23:01:36 cegger Exp $
+/* $Id: line.c,v 1.3 2005/07/30 11:39:57 cegger Exp $
 ******************************************************************************
 
    LibGGI - Millennium II acceleration for fbdev target
@@ -31,9 +31,9 @@
 int GGI_m2164w_drawline(ggi_visual *vis, int x, int y, int x2, int y2)
 {
 	struct m2164w_priv *priv = M2164W_PRIV(vis);
-	volatile uint8 *mmioaddr = FBDEV_PRIV(vis)->mmioaddr;
+	volatile uint8_t *mmioaddr = FBDEV_PRIV(vis)->mmioaddr;
 	int yadd = vis->w_frame_num * LIBGGI_VIRTY(vis);
-	uint32 dwgctl;
+	uint32_t dwgctl;
 
 	if (yadd) {
 		y += yadd;
@@ -52,8 +52,9 @@ int GGI_m2164w_drawline(ggi_visual *vis, int x, int y, int x2, int y2)
 	} else {
 		mga_waitfifo(mmioaddr, 2);
 	}
-	mga_out32(mmioaddr, RS16(x) | (RS16(y) << 16), XYSTRT);
-	mga_out32(mmioaddr, RS16(x2) | (RS16(y2) << 16), XYEND | EXECUTE);
+	mga_out32(mmioaddr, (unsigned)RS16(x) | (RS16(y) << 16), XYSTRT);
+	mga_out32(mmioaddr, (unsigned)RS16(x2) | (RS16(y2) << 16),
+		  XYEND | EXECUTE);
 
 	vis->accelactive = 1;
 
