@@ -1,4 +1,4 @@
-/* $Id: visual.c,v 1.2 2001/10/17 04:33:41 skids Exp $
+/* $Id: visual.c,v 1.6 2005/07/30 11:40:02 cegger Exp $
 ******************************************************************************
 
    Graphics library for GGI.
@@ -29,7 +29,7 @@
 #include "lin4rlib.h"
 
 static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
-			const char *args, void *argptr, uint32 *dlret)
+			const char *args, void *argptr, uint32_t *dlret)
 {
 	/* Color mapping 
 	 */
@@ -75,11 +75,14 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 }
 
 
+EXPORTFUNC
+int GGIdl_linear_4_r(int func, void **funcptr);
+
 int GGIdl_linear_4_r(int func, void **funcptr)
 {
 	switch (func) {
 	case GGIFUNC_open:
-		*funcptr = GGIopen;
+		*funcptr = (void *)GGIopen;
 		return 0;
 	case GGIFUNC_exit:
 	case GGIFUNC_close:
