@@ -1,4 +1,4 @@
-/* $Id: copybox.c,v 1.1.1.1 2001/05/12 23:01:50 cegger Exp $
+/* $Id: copybox.c,v 1.4 2005/07/30 11:40:03 cegger Exp $
 ******************************************************************************
    Graphics library for GGI.
 
@@ -50,17 +50,17 @@ do_copy(ggi_visual *vis, int x, int y, int w, int h, int nx, int ny, void *buf)
 int
 GGI_stubs_copybox(ggi_visual *vis, int x, int y, int w, int h, int nx, int ny)
 {
-	int size;
+	size_t size;
 
 	LIBGGICLIP_COPYBOX(vis, x, y, w, h, nx, ny);
 
-	size = (GT_SIZE(LIBGGI_GT(vis))*w+7)/8;
+	size = GT_ByPPP(w, LIBGGI_GT(vis));
 	if (size <= MAX_STACKBYTES) {
-		uint8 buf[MAX_STACKBYTES];
+		uint8_t buf[MAX_STACKBYTES];
 
 		do_copy(vis, x, y, w, h, nx, ny, buf);
 	} else {
-		uint8 *buf = malloc(size);
+		uint8_t *buf = malloc(size);
 
 		if (!buf) return GGI_ENOMEM;
 		do_copy(vis, x, y, w, h, nx, ny, buf);
