@@ -1,4 +1,4 @@
-/* $Id: visual.c,v 1.1.1.1 2001/05/12 23:01:50 cegger Exp $
+/* $Id: visual.c,v 1.5 2005/07/30 11:40:03 cegger Exp $
 ******************************************************************************
 
    Graphics library for GGI.
@@ -31,7 +31,7 @@
 
 
 static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
-			const char *args, void *argptr, uint32 *dlret)
+			const char *args, void *argptr, uint32_t *dlret)
 {
 	/* Color handling
 	 */
@@ -68,11 +68,14 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 }
 
 
+EXPORTFUNC
+int GGIdl_text_32(int func, void **funcptr);
+
 int GGIdl_text_32(int func, void **funcptr)
 {
 	switch (func) {
 	case GGIFUNC_open:
-		*funcptr = GGIopen;
+		*funcptr = (void *)GGIopen;
 		return 0;
 	case GGIFUNC_exit:
 	case GGIFUNC_close:
