@@ -1,9 +1,10 @@
-/* $Id: ddinit.h,v 1.2 2002/09/08 21:37:45 soyt Exp $
+/* $Id: ddinit.h,v 1.18 2005/01/19 20:55:29 pekberg Exp $
 *****************************************************************************
 
    LibGGI DirectX target - Header for internal functions
 
    Copyright (C) 1999 John Fortin       [fortinj@ibm.net]
+   Copyright (C) 2004 Peter Ekberg      [peda@lysator.liu.se]
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -28,53 +29,35 @@
 #include "config.h"
 #include <ggi/internal/ggi-dl.h>
 #include <ggi/display/directx.h>
+#include <ggi/input/directx.h>
 
 
 #define NAME			"GGI-on-DX"
 #define TITLE			"GGI-on-DX"
 #define WM_DDMESSAGEBOX		0x7FFF
-#define WM_DDPUTPIXEL		0x7FFE
-#define WM_DDDRAWLINE		0x7FFD
-#define WM_DDCHANGEMODE		0x7FFC
-#define MAX_DISPLAYMODES	100
+#define WM_DDCHANGEMODE		0x7FFE
+#define WM_DDEND		0x7FFD
+#define WM_DDFULLSCREEN		0x7FFC
+#define WM_DDSETPALETTE		0x7FFB
 
-typedef struct DisplayMode
+typedef struct directx_fullscreen
 {
-	uint32 width;
-	uint32 height;
-	uint32 bpp;
-} DisplayMode, *LPDisplayMode;
-
-typedef struct DDChangeModeStruct
-{
-	DWORD width;
-	DWORD height;
-	DWORD BPP;
-} DDCMS, *LPDDCMS;
-
-typedef struct DDMessageBoxStruct
-{
-	HWND hWnd;
-	LPCTSTR text;
-	LPCTSTR caption;
-	UINT type;
-} DDMBS, *LPDDMBS;
+	directx_priv *priv;
+	ggi_mode *mode;
+	HRESULT hr;
+	HANDLE event;
+} directx_fullscreen;
 
 
 __BEGIN_DECLS
 
-HANDLE DDInit(directx_priv *);
-DWORD WINAPI DDInitThread(LPVOID lpParm);
-HRESULT redraw(void);
-int DDDrawLine(directx_priv *priv, int x0, int y0, int x1, int y1,
-	       ggi_pixel pix);
-int DDPutPixel(directx_priv *priv, int x, int y, ggi_pixel pix);
-int DXGetPixel(directx_priv *priv, int x, int y, ggi_pixel *color);
-int DXChangeMode(directx_priv *priv, DWORD width, DWORD height, DWORD BPP);
-HRESULT DDChangeMode(directx_priv *priv, DDCMS * ddcms);
-int DDCheckMode(ggi_mode * tm);
-char *DDLock(void);
-HRESULT DDUnlock(void);
-HRESULT DDMessageBox(HWND hWnd, LPCTSTR text, LPCTSTR caption);
+int DDInit(ggi_visual *vis);
+void DDShutdown(directx_priv *priv);
+void DDRedraw(ggi_visual *vis, int x, int y, int w, int h);
+void DDRedrawAll(ggi_visual *vis);
+int DDChangeMode(ggi_visual *vis, ggi_mode *mode);
+int DDMatchMode(ggi_visual *vis, ggi_mode *mode,
+		int *depth, int *defwidth, int *defheight);
+int DDChangePalette(ggi_visual *vis);
 
 __END_DECLS
