@@ -1,4 +1,4 @@
-/* $Id: visual.c,v 1.7 2001/08/24 01:39:40 skids Exp $
+/* $Id: visual.c,v 1.31 2005/09/19 18:46:41 cegger Exp $
 ******************************************************************************
 
    Display-FBDEV: visual handling
@@ -26,9 +26,11 @@
 ******************************************************************************
 */
 
-#include <ggi/internal/ggi-dl.h>
+#include "config.h"
 #include <ggi/display/fbdev.h>
 #include <ggi/display/linvtsw.h>
+#include <ggi/internal/ggi_debug.h>
+#include <ggi/internal/gg_replace.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -110,76 +112,19 @@ static const gg_option optlist[] =
 #define MAX_DEV_LEN	63
 #define DEFAULT_FBNUM	0
 
-#ifdef HAVE_NEW_FBDEV
-static char accel_prefix[] = "tgt-fbdev-";
-#define PREFIX_LEN	(sizeof(accel_prefix))
-
-typedef struct {
-	char *str;
-	int   async;
-	int   flags;
-} accel_info;
-
-
-static accel_info accel_strings[] = {
-/* 0  */{ "kgicon-generic", 0, 0 },	/* no accel - check for KGIcon	*/
-	{ NULL, 0, 0 },			/* Atari Blitter		*/
-	{ NULL, 0, 0 },			/* Amiga Blitter                */
-	{ NULL, 0, 0 },			/* Cybervision64 (S3 Trio64)    */
-	{ NULL, 0, 0 },			/* RetinaZ3 (NCR 77C32BLT)      */
-	{ NULL, 0, 0 },			/* Cybervision64/3D (S3 ViRGE)	*/
-	{ NULL, 0, 0 },			/* ATI Mach 64GX family		*/
-	{ NULL, 0, 0 },			/* DEC 21030 TGA		*/
-	{ NULL, 0, 0 },			/* ATI Mach 64CT family		*/
-	{ NULL, 0, 0 },			/* ATI Mach 64CT family VT class */
-/* 10 */{ NULL, 0, 0 },			/* ATI Mach 64CT family GT class */
-	{ NULL, 0, 0 },			/* Sun Creator/Creator3D	*/
-	{ NULL, 0, 0 },			/* Sun cg6			*/
-	{ NULL, 0, 0 },			/* Sun leo/zx			*/
-	{ NULL, 0, 0 },			/* IMS Twin Turbo		*/
-	{ NULL, 0, 0 },			/* 3Dlabs Permedia 2		*/
-	{ "mga-2164w", 1, GGI_FBDEV_4BPP_REV 
-	},				/* Matrox MGA2064W (Millenium)	*/
-	{ "directfb", 1, 0
-	},				/* Matrox MGA1064SG (Mystique)	*/
-	{ "mga-2164w", 1, GGI_FBDEV_4BPP_REV 
-	},	   			/* Matrox MGA2164W (Millenium II) */
-	{ "mga-2164w", 1, GGI_FBDEV_4BPP_REV 
-	},	   			/* Matrox MGA2164W (Millenium II AGP)*/
-/* 20 */ 
-	{ "mga-2164w", 1, GGI_FBDEV_4BPP_REV 
-	},	        		/* Matrox G100 (Productiva G100) */
-	{ "mga-2164w", 1, GGI_FBDEV_4BPP_REV 
-	},	        		/* Matrox G200 (Myst, Mill, ...) */
-	{ NULL, 0, 0 },			/* Sun cgfourteen		*/
-	{ NULL, 0, 0 },			/* Sun bwtwo			*/
-	{ NULL, 0, 0 },			/* Sun cgthree			*/
-	{ NULL, 0, 0 },			/* Sun tcx			*/
-	{ "mga-g400", 1, GGI_FBDEV_4BPP_REV
-	},				/* Matrox G400			*/
-	{ "directfb", 1, 0 },		/* nVidia RIVA 128              */
-	{ "directfb", 1, 0 },		/* nVidia RIVA TNT              */
-	{ "directfb", 1, 0 },		/* nVidia RIVA TNT2             */
-/* 30 */{ NULL, 0, 0 },			/* C&T 6555x			*/
-	{ "directfb", 1, 0 },		/* 3Dfx Banshee			*/
-	{ "directfb", 1, 0 }		/* ATI Rage128 family		*/
-};
-
-#define NUM_ACCELS	(sizeof(accel_strings)/sizeof(accel_info))
-#endif /* HAVE_NEW_FBDEV */
-
-
-extern int GGI_fbdev_resetmode(ggi_visual *vis);
+extern int GGI_fbdev_mode_reset(ggi_visual *vis);
+extern void GGI_fbdev_color0(ggi_visual *vis);
+extern void GGI_fbdev_color_free(ggi_visual *vis);
 
 static void
 switchreq(void *arg)
 {
 	ggi_visual *vis = arg;
-	ggi_fbdev_priv *priv = LIBGGI_PRIVATE(vis);
+	ggi_fbdev_priv *priv = FBDEV_PRIV(vis);
 	gii_event ev;
 	ggi_cmddata_switchrequest *data;
 
-	GGIDPRINT_MISC("display-fbdev: switchreq(%p) called\n", vis);
+	DPRINT_MISC("display-fbdev: switchreq(%p) called\n", vis);
 
 	_giiEventBlank(&ev, sizeof(gii_cmd_event));
 
@@ -200,9 +145,9 @@ static void
 switching(void *arg)
 {
 	ggi_visual *vis = arg;
-	ggi_fbdev_priv *priv = LIBGGI_PRIVATE(vis);
+	ggi_fbdev_priv *priv = FBDEV_PRIV(vis);
 
-	GGIDPRINT_MISC("display-fbdev: switching(%p) called\n", vis);
+	DPRINT_MISC("display-fbdev: switching(%p) called\n", vis);
 
 	priv->ismapped = 0;
 	priv->switchpending = 0;
@@ -215,42 +160,6 @@ switching(void *arg)
 #endif
 }
 
-
-static void
-do_setpalette(ggi_visual *vis)
-{
-	ggi_graphtype gt = LIBGGI_MODE(vis)->graphtype;
-	ggi_fbdev_priv *priv = LIBGGI_PRIVATE(vis);
-	ggi_color *colormap;
-	struct fb_cmap cmap;
-	int len, i;
-
-	if (GT_SCHEME(gt) != GT_PALETTE) return;
-
-	len = 1 << GT_DEPTH(gt);
-	cmap.start  = 0;
-	cmap.len    = len;
-	cmap.red    = priv->reds;
-	cmap.green  = priv->greens;
-	cmap.blue   = priv->blues;
-	cmap.transp = NULL;
-	colormap = vis->palette;
-
-	for (i = 0; i < len; i++) {
-		priv->reds[i]   = colormap[i].r;
-		priv->greens[i] = colormap[i].g;
-		priv->blues[i]  = colormap[i].b;
-	}
-
-	if (fbdev_doioctl(vis, FBIOPUTCMAP, &cmap) < 0) {
-		GGIDPRINT_COLOR("display-fbdev: PUTCMAP failed.");
-	} else {
-		GGIDPRINT_COLOR("display-fbdev: restored palette for %p\n",
-				vis);
-	}
-}
-
-
 static void
 switchback(void *arg)
 {
@@ -258,7 +167,7 @@ switchback(void *arg)
 	ggi_fbdev_priv *priv = FBDEV_PRIV(vis);
 	gii_event ev;
 
-	GGIDPRINT_MISC("display-fbdev: switched_back(%p) called\n", vis);
+	DPRINT_MISC("display-fbdev: switched_back(%p) called\n", vis);
 
 	_giiEventBlank(&ev, sizeof(gii_expose_event));
 
@@ -270,14 +179,22 @@ switchback(void *arg)
 	ev.expose.h = LIBGGI_VIRTY(vis);
 
 	_giiSafeAdd(vis->input, &ev);
-	GGIDPRINT_MISC("fbdev: EXPOSE sent.\n");
+	DPRINT_MISC("fbdev: EXPOSE sent.\n");
 
 #if 0
 	if (refcount > 1) {
 		fbdev_doioctl(vis, FBIOPUT_VSCREENINFO, &priv->var);
 	}
 #endif
-	do_setpalette(vis);
+
+	/* See notes about palette member reuse in color.c */
+	if ((LIBGGI_PAL(vis)->setPalette != NULL) && (vis->opcolor->setpalvec != NULL))
+		vis->opcolor->setpalvec(vis, 0, 1 << GT_DEPTH(LIBGGI_GT(vis)), 
+					LIBGGI_PAL(vis)->clut.data);
+	else if (vis->opcolor->setgammamap != NULL)
+		vis->opcolor->setgammamap(vis, 0, vis->gamma->len, 
+					  LIBGGI_PAL(vis)->clut.data);
+
 	if (priv->fix.xpanstep != 0 || priv->fix.ypanstep != 0) {
 		fbdev_doioctl(vis, FBIOPAN_DISPLAY, &priv->var);
 	}
@@ -288,16 +205,16 @@ switchback(void *arg)
 static int 
 GGI_fbdev_sendevent(ggi_visual *vis, gii_event *ev)
 {
-	ggi_fbdev_priv *priv = LIBGGI_PRIVATE(vis);
+	ggi_fbdev_priv *priv = FBDEV_PRIV(vis);
 
-	GGIDPRINT_MISC("GGI_fbdev_sendevent() called\n");
+	DPRINT_MISC("GGI_fbdev_sendevent() called\n");
 
 	if (ev->any.type != evCommand) {
 		return GGI_EEVUNKNOWN;
 	}
 	switch (ev->cmd.code) {
 	case GGICMD_ACKNOWLEDGE_SWITCH:
-		GGIDPRINT_MISC("display-fbdev: switch acknowledge\n");
+		DPRINT_MISC("display-fbdev: switch acknowledge\n");
 		if (priv->switchpending) {
 			priv->doswitch(vis);
 			return 0;
@@ -307,12 +224,12 @@ GGI_fbdev_sendevent(ggi_visual *vis, gii_event *ev)
 		}
 		break;
 	case GGICMD_NOHALT_ON_UNMAP:
-		GGIDPRINT_MISC("display-fbdev: nohalt on\n");
+		DPRINT_MISC("display-fbdev: nohalt on\n");
 		priv->dohalt = 0;
 		priv->autoswitch = 0;
 		break;
 	case GGICMD_HALT_ON_UNMAP:
-		GGIDPRINT_MISC("display-fbdev: halt on\n");
+		DPRINT_MISC("display-fbdev: halt on\n");
 		priv->dohalt = 1;
 		priv->autoswitch = 1;
 		if (priv->switchpending) {
@@ -342,85 +259,22 @@ GGI_fbdev_idleaccel(ggi_visual *vis)
 {
 	ggi_fbdev_priv *priv = FBDEV_PRIV(vis);
 
-	GGIDPRINT_DRAW("GGI_fbdev_idleaccel(%p) called \n", vis);
+	DPRINT_DRAW("GGI_fbdev_idleaccel(%p) called \n", vis);
 	
 	if (priv->idleaccel) return priv->idleaccel(vis);
 
 	return 0;
 }
 
-
-static void 
-save_palette(ggi_visual *vis)
-{
-	ggi_fbdev_priv *priv = LIBGGI_PRIVATE(vis);
-	struct fb_cmap cmap;
-	int len;
-
-	if ((priv->orig_var.bits_per_pixel < 1) || 
-	    (priv->orig_var.bits_per_pixel > 8)) {
-		priv->orig_reds = NULL;
-		return;
-	}
-	len = 1 << priv->orig_var.bits_per_pixel;
-	
-	priv->orig_reds = malloc(sizeof(uint16)*len*3);
-	if (priv->orig_reds == NULL) {
-		return;
-	}
-
-	cmap.start = 0;
-	cmap.len   = len;
-	cmap.red   = priv->orig_reds;
-	cmap.green = priv->orig_greens = priv->orig_reds + len;
-	cmap.blue  = priv->orig_blues  = priv->orig_greens + len;
-	cmap.transp = NULL;
-
-	if (ioctl(LIBGGI_FD(vis), FBIOGETCMAP, &cmap) < 0) {
-		GGIDPRINT_COLOR("display-fbdev: GETCMAP failed.\n");
-	} else {
-		GGIDPRINT_COLOR("display-fbdev: Saved palette (len=%d).\n",
-				cmap.len);
-	}
-}
-
-
-static void 
-restore_palette(ggi_visual *vis)
-{
-	ggi_fbdev_priv *priv = LIBGGI_PRIVATE(vis);
-	struct fb_cmap cmap;
-
-	if (priv->orig_reds == NULL) return;
-	
-	cmap.start  = 0;
-	cmap.len    = 1 << priv->orig_var.bits_per_pixel;
-	cmap.red    = priv->orig_reds;
-	cmap.green  = priv->orig_greens;
-	cmap.blue   = priv->orig_blues;
-	cmap.transp = NULL;
-
-	if (ioctl(LIBGGI_FD(vis), FBIOPUTCMAP, &cmap) < 0) {
-		GGIDPRINT_COLOR("display-fbdev: PUTCMAP failed.\n");
-	} else {
-		GGIDPRINT_COLOR("display-fbdev: Restored palette (len=%d).\n", 
-				cmap.len);
-	}
-
-	free(priv->orig_reds);
-	priv->orig_reds = NULL;
-}
-
-
 static int do_cleanup(ggi_visual *vis)
 {
-	ggi_fbdev_priv *priv = LIBGGI_PRIVATE(vis);
+	ggi_fbdev_priv *priv = FBDEV_PRIV(vis);
 	ggi_fbdev_timing *curtim;
 
 	/* We may be called more than once due to the LibGG cleanup stuff */
 	if (priv == NULL) return 0;
 
-	GGIDPRINT("display-fbdev: GGIdlcleanup start.\n");
+	DPRINT("display-fbdev: GGIdlcleanup start.\n");
 
 	if (LIBGGI_FD(vis) >= 0) {
 #if 0
@@ -428,9 +282,8 @@ static int do_cleanup(ggi_visual *vis)
 		ioctl(LIBGGI_FD(vis), FBIOPUT_CON2FBMAP, &origconmap);
 #endif
 #endif
-		GGI_fbdev_resetmode(vis);
-		restore_palette(vis);
-		close(LIBGGI_FD(vis));
+		GGI_fbdev_color_free(vis);
+		GGI_fbdev_mode_reset(vis);
 	}
 
 	if (vis->input != NULL) {
@@ -441,9 +294,6 @@ static int do_cleanup(ggi_visual *vis)
 	if (priv->normalgc) {
 		free(priv->normalgc);
 	}
-	if (priv->accel) {
-		free(priv->accel);
-	}
 	curtim = priv->timings;
 	while (curtim) {
 		ggi_fbdev_timing *prevtim = curtim;
@@ -464,7 +314,7 @@ static int do_cleanup(ggi_visual *vis)
 	}
 	ggUnlock(_ggi_global_lock);
 
-	GGIDPRINT("display-fbdev: GGIdlcleanup done.\n");
+	DPRINT("display-fbdev: GGIdlcleanup done.\n");
 
 	return 0;
 }
@@ -489,7 +339,7 @@ get_fbdev(void)
 
 	if (ioctl(fd, VT_GETSTATE, &qry_stat) == 0) {
 		map.console = qry_stat.v_active;
-		GGIDPRINT_MISC("display-fbdev: Using VT %d.\n", map.console);
+		DPRINT_MISC("display-fbdev: Using VT %d.\n", map.console);
 	} else {
 		perror("display-fbdev: ioctl(VT_GETSTATE) failed");
 		close(fd);
@@ -499,17 +349,18 @@ get_fbdev(void)
 
 	/* Find a framebuffer to open */
 	for (fb=0; fb < 32; fb++) {
-		sprintf(devname, "/dev/fb%d", fb);
+		snprintf(devname, MAX_DEV_LEN, "/dev/fb%d", fb);
 		fd = open(devname, O_RDONLY);
 		if (fd >= 0) break;
-		sprintf(devname, "/dev/fb/%d", fb);
+
+		snprintf(devname, MAX_DEV_LEN, "/dev/fb/%d", fb);
 		fd = open(devname, O_RDONLY);
 		if (fd >= 0) break;
 	}
 
 	if (fb >= 32) {
 		/* Not found */
-		GGIDPRINT_MISC("display-fbdev: Could not find a framebuffer "
+		DPRINT_MISC("display-fbdev: Could not find a framebuffer "
 			       "device with read permission.\n");
 		return DEFAULT_FBNUM;
 	}
@@ -522,7 +373,7 @@ get_fbdev(void)
 
 	close(fd);
 
-	GGIDPRINT_MISC("display-fbdev: Determined VT %d is on FB %d\n",
+	DPRINT_MISC("display-fbdev: Determined VT %d is on FB %d\n",
 		       map.console, map.framebuffer);
 
 	return map.framebuffer;
@@ -567,13 +418,13 @@ get_timings(ggi_fbdev_priv *priv, const char *name)
 	 */
 	infile = fopen(name, "r");
 	if (infile == NULL) {
-		GGIDPRINT_MODE("display-fbdev: Unable to open: %s\n", name);
+		DPRINT_MODE("display-fbdev: Unable to open: %s\n", name);
 		return GGI_ENOFILE;
 	}
 
 	/* Scan for all modes.
 	 */
-	GGIDPRINT_MODE("display-fbdev: Parsing modedb: %s\n", name);
+	DPRINT_MODE("display-fbdev: Parsing modedb: %s\n", name);
 	while (!feof(infile)) {
 		/* EOF ? */
 		if (NULL == fgets(buffer, sizeof(buffer), infile)) break;
@@ -590,17 +441,17 @@ get_timings(ggi_fbdev_priv *priv, const char *name)
 		 */
 		if (0 == strncmp(bufp, "mode", 4)) {
 			memset(&timing, 0, sizeof(timing));
-			GGIDPRINT_MODE("display-fbdev:   begin: %s",
+			DPRINT_MODE("display-fbdev:   begin: %s",
 				       bufp+4);
 			continue;
 		} else if (0 == strncmp(bufp, "geometry", 8)) {
-			sscanf(bufp+8, " %d %d %d %d %d",
+			sscanf(bufp+8, " %u %u %u %u %u",
 			       &timing.xres, &timing.yres,
 			       &timing.xres_virtual, &timing.yres_virtual,
 			       &timing.bits_per_pixel);
 			continue;
 		} else if (0 == strncmp(bufp, "timings", 7)) {
-			sscanf(bufp+7, " %d %d %d %d %d %d %d",
+			sscanf(bufp+7, " %u %u %u %u %u %u %u",
 			       &timing.pixclock,
 			       &timing.left_margin, &timing.right_margin,
 			       &timing.upper_margin, &timing.lower_margin,
@@ -665,7 +516,7 @@ get_timings(ggi_fbdev_priv *priv, const char *name)
 		} else if (0 == strncmp(bufp, "endmode", 7) &&
 			   timing.pixclock /* Valid timing entry ? */ &&
 			   timing.xres && timing.yres) {
-			GGIDPRINT_MODE("display-fbdev:   got %dx%d mode\n",
+			DPRINT_MODE("display-fbdev:   got %dx%d mode\n",
 				       timing.xres, timing.yres);
 			curtim->next = malloc(sizeof(timing));
 			if (!curtim->next) {
@@ -686,7 +537,7 @@ get_timings(ggi_fbdev_priv *priv, const char *name)
 #define MAX_DEV_LEN	63
 
 static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
-			const char *args, void *argptr, uint32 *dlret)
+			const char *args, void *argptr, uint32_t *dlret)
 {
 	ggi_linvtsw_arg vtswarg;
 	const char *devfile = NULL;
@@ -695,11 +546,11 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 	int   novt = 0;
 	int   classicname_failed = 0;
 	int   classicname_errno = 0;	/* Keep compiler happy. */
-	char *modedb = DEFAULT_MODEDB;
+	const char *modedb = DEFAULT_MODEDB;
 	gg_option options[NUM_OPTS];
 	ggi_fbdev_priv *priv;
 
-	GGIDPRINT("display-fbdev: GGIdlinit start.\n");
+	DPRINT("display-fbdev: GGIdlinit start.\n");
 
 	memcpy(options, optlist, sizeof(options));
 
@@ -721,18 +572,17 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 		}
 	}
 
-	LIBGGI_PRIVATE(vis) = priv = malloc(sizeof(ggi_fbdev_priv));
+	priv = calloc(1, sizeof(ggi_fbdev_priv));
 	if (priv == NULL) {
 		return GGI_ENOMEM;
 	}
-	GGIDPRINT("display-fbdev: Got private mem.\n");
-	
+	DPRINT("display-fbdev: Got private mem.\n");
+	LIBGGI_PRIVATE(vis) = priv;
+
 	priv->fb_ptr = NULL;
 	priv->need_timings = 1;
 	priv->flags = 0;
 
-	priv->orig_reds = NULL;
-
 	priv->dohalt = 1;
 	priv->autoswitch = 1;
 	priv->switchpending = 0;
@@ -740,7 +590,6 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 	priv->doswitch = NULL;
 
 	priv->iskgi = 0;
-	priv->accel = NULL;
 	priv->have_accel = 0;
 	priv->accelpriv = NULL;
 	priv->mmioaddr = NULL;
@@ -750,36 +599,34 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 	if (strlen(options[OPT_DEV].result)) {
 		devfile = options[OPT_DEV].result;
 	} else if (getenv("FRAMEBUFFER") != NULL) {
-		strncpy(devicename, getenv("FRAMEBUFFER"), MAX_DEV_LEN);
-		/* Make sure string is terminated */
-		devicename[MAX_DEV_LEN] = '\0';
+		ggstrlcpy(devicename, getenv("FRAMEBUFFER"), sizeof(devicename));
 		devfile = devicename;
 	} else {
 		fbnum = get_fbdev();
 	}
 
-	GGIDPRINT("display-fbdev: Parsing input options.\n");
-	
+	DPRINT("display-fbdev: Parsing input options.\n");
+
 	priv->inputs = FBDEV_INP_KBD | FBDEV_INP_MOUSE;
 
-	if (toupper(options[OPT_NOKBD].result[0]) != 'N') {
+	if (toupper((uint8_t)options[OPT_NOKBD].result[0]) != 'N') {
 		priv->inputs &= ~FBDEV_INP_KBD;
 	}
-	if (toupper(options[OPT_NOMOUSE].result[0]) != 'N') {
+	if (toupper((uint8_t)options[OPT_NOMOUSE].result[0]) != 'N') {
 		priv->inputs &= ~FBDEV_INP_MOUSE;
 	}
-	if (toupper(options[OPT_NOINPUT].result[0]) != 'N') {
+	if (toupper((uint8_t)options[OPT_NOINPUT].result[0]) != 'N') {
 		priv->inputs &= ~(FBDEV_INP_KBD | FBDEV_INP_MOUSE);
 	}
-	if (toupper(options[OPT_NOVT].result[0]) != 'N') {
+	if (toupper((uint8_t)options[OPT_NOVT].result[0]) != 'N') {
 		priv->inputs = 0;
 		novt = 1;
 	}
 
-	GGIDPRINT("display-fbdev: Parsing physz options.\n");
+	DPRINT("display-fbdev: Parsing physz options.\n");
 	do {
 		int err;
-		err = _ggi_parse_physz(options[OPT_PHYSZ].result, 
+		err = _ggi_physz_parse_option(options[OPT_PHYSZ].result, 
 				       &(priv->physzflags), &(priv->physz)); 
 		if (err != GGI_OK) {
 			do_cleanup(vis);
@@ -787,7 +634,7 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 		}
 	} while (0);
 
-	GGIDPRINT("display-fbdev: Setting up locks.\n");
+	DPRINT("display-fbdev: Setting up locks.\n");
 	ggLock(_ggi_global_lock);
 	if (refcount == 0) {
 		_ggi_fbdev_lock = ggLockCreate();
@@ -818,7 +665,8 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 		}
 		vtswarg.novt = novt;
 
-		if (_ggiAddDL(vis, "helper-linux-vtswitch", NULL, &vtswarg, 0)
+		if (_ggiAddDL(vis, _ggiGetConfigHandle(),
+				"helper-linux-vtswitch", NULL, &vtswarg, 0)
 		    == 0) {
 			vtnum = vtswarg.vtnum;
 			priv->doswitch = vtswarg.doswitch;
@@ -836,7 +684,7 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 	/* Open keyboard and mouse input */
 	if (priv->inputs & FBDEV_INP_KBD) {
 		char strbuf[64];
-		char *inputstr = "input-linux-kbd";
+		const char *inputstr = "input-linux-kbd";
 
 		if (vtnum != -1) {
 			sprintf(strbuf, "linux-kbd:/dev/tty%d", vtnum);
@@ -880,7 +728,7 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 			char *strptr = devicename;
 			char *chkptr;
 
-			while (!isdigit(*strptr)) {
+			while (!isdigit((uint8_t)*strptr)) {
 				if (*strptr == '\0') break;
 				strptr++;
 			}
@@ -939,13 +787,13 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 		if (rc == 0 && iskgi== KGICON_PROBE_MAGIC) {
 			struct kgi_driver info;
 
-			GGIDPRINT_MISC("display-fbdev: Detected KGI driver.\n");
+			DPRINT_MISC("display-fbdev: Detected KGI driver.\n");
 			priv->iskgi = 1;
 			priv->need_timings = 0;
 
 			rc = ioctl(LIBGGI_FD(vis), DRIVER_GETINFO, &info);
 			if (!rc) {
-				GGIDPRINT_MISC("display-fbdev: KGI driver by %s, type %s, version %08x.\n",
+				DPRINT_MISC("display-fbdev: KGI driver by %s, type %s, version %08x.\n",
 					       info.manufact, info.model,
 					       info.version);
 			}
@@ -961,7 +809,6 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 		do_cleanup(vis);
 		return GGI_ENODEVICE;
 	}
-	save_palette(vis);
 
 	LIBGGI_GC(vis) = priv->normalgc = malloc(sizeof(ggi_gc));
 	if (priv->normalgc == NULL) {
@@ -969,36 +816,23 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 		return GGI_ENOMEM;
 	}
 
-#ifdef HAVE_NEW_FBDEV
-	GGIDPRINT_MISC("display-fbdev: accel: %d, supported: %d\n",
-		       priv->orig_fix.accel, NUM_ACCELS);
-	if (priv->orig_fix.accel >= 0 && priv->orig_fix.accel < NUM_ACCELS) {
-		vis->needidleaccel = accel_strings[priv->orig_fix.accel].async;
-		priv->flags |= accel_strings[priv->orig_fix.accel].flags;
-		if (accel_strings[priv->orig_fix.accel].str == NULL) {
-			goto no_accel_lib;
-		}
-		priv->accel
-			= malloc(strlen(accel_strings[priv->orig_fix.accel].str)
-				 + PREFIX_LEN);
-		if (priv->accel == NULL) {
-			do_cleanup(vis);
-			return GGI_ENOMEM;
-		}
-		sprintf(priv->accel, "%s%s", accel_prefix,
-			accel_strings[priv->orig_fix.accel].str);
-		GGIDPRINT_MISC("display-fbdev: Have accel: \"%s\"\n",
-			       accel_strings[priv->orig_fix.accel].str);
-	}
-  no_accel_lib:
-#endif /* HAVE_NEW_FBDEV */
-
-	if (get_timings(priv, modedb) == GGI_ENOMEM) {
+	switch (get_timings(priv, modedb)) {
+	case GGI_OK:
+		break;
+	case GGI_ENOFILE:
+		/* PDA's like Compaq iPaq and Sharp Zaurus don't
+		 * need /etc/fb.modes to work, so we ignore this failure.
+		 */
+		break;
+	case GGI_ENOMEM:
 		/* If we can't do a simple malloc() it's better to fail
-		   right now. */
+		 * right now.
+		 */
 		do_cleanup(vis);
 		return GGI_ENOMEM;
-	}
+	default:
+		break;
+	}	/* switch */
 
 	/* Mode management */
 	vis->opdisplay->getmode   = GGI_fbdev_getmode;
@@ -1014,7 +848,7 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 	/* Register cleanup handler */
 	ggRegisterCleanup((ggcleanup_func *)do_cleanup, vis);
 
-	GGIDPRINT("display-fbdev: GGIdlinit success.\n");
+	DPRINT("display-fbdev: GGIdlinit success.\n");
 
 	*dlret = GGI_DL_OPDISPLAY;
 	return 0;
@@ -1026,17 +860,20 @@ static int GGIclose(ggi_visual *vis, struct ggi_dlhandle *dlh)
 }
 
 
+EXPORTFUNC
+int GGIdl_fbdev(int func, void **funcptr);
+
 int GGIdl_fbdev(int func, void **funcptr)
 {
 	switch (func) {
 	case GGIFUNC_open:
-		*funcptr = GGIopen;
+		*funcptr = (void *)GGIopen;
 		return 0;
 	case GGIFUNC_exit:
 		*funcptr = NULL;
 		return 0;
 	case GGIFUNC_close:
-		*funcptr = GGIclose;
+		*funcptr = (void *)GGIclose;
 		return 0;
 	default:
 		*funcptr = NULL;
