@@ -1,4 +1,4 @@
-/* $Id: mode.c,v 1.3 2002/10/27 18:26:24 skids Exp $
+/* $Id: mode.c,v 1.12 2005/09/19 18:46:42 cegger Exp $
 ******************************************************************************
 
    Display-lcd823
@@ -59,7 +59,7 @@ int GGI_lcd823_getapi(ggi_visual *vis, int num, char *apiname, char *arguments)
 {
 	int size = GT_SIZE(LIBGGI_GT(vis));
 
-	strcpy(arguments, "");
+	*arguments = '\0';
 
 	switch(num) {
 	case 0: strcpy(apiname, "display-lcd823");
@@ -81,13 +81,13 @@ int GGI_lcd823_getapi(ggi_visual *vis, int num, char *apiname, char *arguments)
 		return 0;
 	}
 
-	return -1;
+	return GGI_ENOMATCH;
 }
 
 
 static int do_mmap(ggi_visual *vis)
 {
-	ggi_lcd823_priv *priv = LIBGGI_PRIVATE(vis);
+	ggi_lcd823_priv *priv = LCD823_PRIV(vis);
 	ggi_graphtype gt = LIBGGI_GT(vis);
 	int xres_in_bytes = XRES;
 	int i;
@@ -112,7 +112,7 @@ static int do_mmap(ggi_visual *vis)
 
 		buf->frame = i;
 		buf->type  = GGI_DB_NORMAL;
-		buf->read  = (uint8 *) priv->fb_ptr + i * priv->frame_size;
+		buf->read  = (uint8_t *) priv->fb_ptr + i * priv->frame_size;
 		buf->write = buf->read;
 
 		buf->type  |= GGI_DB_SIMPLE_PLB;
@@ -127,7 +127,7 @@ static int do_mmap(ggi_visual *vis)
 
 static int do_setmode(ggi_visual *vis)
 {
-	char libname[256], libargs[256];
+	char libname[GGI_MAX_APILEN], libargs[GGI_MAX_APILEN];
 	ggi_graphtype gt;
 	int err, id;
 
@@ -136,13 +136,13 @@ static int do_setmode(ggi_visual *vis)
 
 	_ggiZapMode(vis, 0);
 	for (id=1; GGI_lcd823_getapi(vis, id, libname, libargs) == 0; id++) {
-		if (_ggiOpenDL(vis, libname, libargs, NULL) != 0) {
+		if (_ggiOpenDL(vis, _ggiGetConfigHandle(), libname, libargs, NULL) != 0) {
 			fprintf(stderr,"display-lcd823: Error opening the "
 				"%s (%s) library.\n", libname, libargs);
 			return GGI_EFATAL;
 		}
 
-		GGIDPRINT_LIBS("Success in loading %s (%s)\n",
+		DPRINT_LIBS("Success in loading %s (%s)\n",
 			       libname, libargs);
 	}
 
@@ -151,15 +151,18 @@ static int do_setmode(ggi_visual *vis)
 	if ((GT_SCHEME(gt) == GT_PALETTE) || (GT_SCHEME(gt) == GT_TEXT)) {
 	    	int nocols = 1 << GT_DEPTH(gt);
 
-		vis->palette = _ggi_malloc(nocols * sizeof(ggi_color));
-		vis->opcolor->setpalvec = GGI_lcd823_setpalvec;
+		LIBGGI_PAL(vis)->clut.size = nocols;
+		LIBGGI_PAL(vis)->clut.data = _ggi_malloc(nocols * sizeof(ggi_color));
+		LIBGGI_PAL(vis)->priv = _ggi_malloc(256 * sizeof(uint16_t));
+		LIBGGI_PAL(vis)->setPalette  = GGI_lcd823_setPalette;
+		LIBGGI_PAL(vis)->getPrivSize = GGI_lcd823_getPrivSize;
 		/* Initialize palette */
 		ggiSetColorfulPalette(vis);
 	}
 
 	ggiIndicateChange(vis, GGI_CHG_APILIST);
 
-	GGIDPRINT_MODE("display-lcd823: do_setmode SUCCESS\n");
+	DPRINT_MODE("display-lcd823: do_setmode SUCCESS\n");
 
 	return 0;
 }
@@ -173,9 +176,14 @@ int GGI_lcd823_setmode(ggi_visual *vis, ggi_mode *mode)
 		return err;
 	}
 
-	if (vis->palette) {
-		free(vis->palette);
-		vis->palette = NULL;
+	if (LIBGGI_PAL(vis)->clut.data) {
+		free(LIBGGI_PAL(vis)->clut.data);
+		LIBGGI_PAL(vis)->clut.data = NULL;
+	}
+
+	if (LIBGGI_PAL(vis)->priv) {
+		free(LIBGGI_PAL(vis)->priv);
+		LIBGGI_PAL(vis)->priv = NULL;
 	}
 
 	_GGI_lcd823_free_dbs(vis);
@@ -199,7 +207,7 @@ int GGI_lcd823_checkmode(ggi_visual *vis, ggi_mode *mode)
 {
 	int err = 0;
 
-	GGIDPRINT_MODE("display-lcd823: checkmode %dx%d#%dx%dF%d[0x%02x]\n",
+	DPRINT_MODE("display-lcd823: checkmode %dx%d#%dx%dF%d[0x%02x]\n",
 			mode->visible.x, mode->visible.y,
 			mode->virt.x, mode->virt.y, 
 			mode->frames, mode->graphtype);
@@ -239,7 +247,7 @@ int GGI_lcd823_checkmode(ggi_visual *vis, ggi_mode *mode)
 	}
 	mode->size.x = mode->size.y = GGI_AUTO;
 
-	GGIDPRINT_MODE("display-lcd823: result %d %dx%d#%dx%dF%d[0x%02x]\n",
+	DPRINT_MODE("display-lcd823: result %d %dx%d#%dx%dF%d[0x%02x]\n",
 		       err, mode->visible.x, mode->visible.y,
 		       mode->virt.x, mode->virt.y, 
 		       mode->frames, mode->graphtype);
@@ -250,7 +258,7 @@ int GGI_lcd823_checkmode(ggi_visual *vis, ggi_mode *mode)
 
 int GGI_lcd823_getmode(ggi_visual *vis, ggi_mode *mode)
 {
-	GGIDPRINT_MODE("display-lcd823: getmode\n");
+	DPRINT_MODE("display-lcd823: getmode\n");
 
 	memcpy(mode, LIBGGI_MODE(vis), sizeof(ggi_mode));
 
