@@ -1,4 +1,4 @@
-/* $Id: visual.c,v 1.2 2002/09/08 21:37:46 soyt Exp $
+/* $Id: visual.c,v 1.7 2005/07/30 11:38:51 cegger Exp $
 ******************************************************************************
 
    Display-lcd823: visual handling
@@ -58,12 +58,12 @@ GGI_lcd823_flush(ggi_visual *vis, int x, int y, int w, int h, int tryflag)
 
 static int do_cleanup(ggi_visual *vis)
 {
-	ggi_lcd823_priv *priv = LIBGGI_PRIVATE(vis);
+	ggi_lcd823_priv *priv = LCD823_PRIV(vis);
 
 	/* We may be called more than once due to the LibGG cleanup stuff */
 	if (priv == NULL) return 0;
 
-	GGIDPRINT("display-lcd823: do_cleanup start.\n");
+	DPRINT("display-lcd823: do_cleanup start.\n");
 
 	_GGI_lcd823_free_dbs(vis);
 
@@ -82,7 +82,7 @@ static int do_cleanup(ggi_visual *vis)
 	}
 
 	free(priv);
-	LIBGGI_PRIVATE(vis) = NULL;
+	LCD823_PRIV(vis) = NULL;
 
 	if (LIBGGI_GC(vis)) {
 		free(LIBGGI_GC(vis));
@@ -90,7 +90,7 @@ static int do_cleanup(ggi_visual *vis)
 
 	ggUnregisterCleanup((ggcleanup_func *)do_cleanup, vis);
 
-	GGIDPRINT("display-lcd823: do_cleanup done.\n");
+	DPRINT("display-lcd823: do_cleanup done.\n");
 
 	return 0;
 }
@@ -99,14 +99,14 @@ static int do_cleanup(ggi_visual *vis)
 #define MAX_DEV_LEN	63
 
 static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
-		   const char *args, void *argptr, uint32 *dlret)
+		   const char *args, void *argptr, uint32_t *dlret)
 {
 	ggi_lcd823_priv *priv;
 	int size;
 
-	GGIDPRINT("display-lcd823: GGIopen start.\n");
+	DPRINT("display-lcd823: GGIopen start.\n");
 
-	LIBGGI_PRIVATE(vis) = priv = malloc(sizeof(ggi_lcd823_priv));
+	LCD823_PRIV(vis) = priv = malloc(sizeof(ggi_lcd823_priv));
 	if (priv == NULL) {
 		return GGI_ENOMEM;
 	}
@@ -166,7 +166,7 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 	/* Register cleanup handler */
 	ggRegisterCleanup((ggcleanup_func *)do_cleanup, vis);
 
-	GGIDPRINT("display-lc823: GGIopen success.\n");
+	DPRINT("display-lc823: GGIopen success.\n");
 
 	*dlret = GGI_DL_OPDISPLAY;
 	return 0;
@@ -178,17 +178,20 @@ static int GGIclose(ggi_visual *vis, struct ggi_dlhandle *dlh)
 }
 
 
+EXPORTFUNC
+int GGIdl_lcd823(int func, void **funcptr);
+
 int GGIdl_lcd823(int func, void **funcptr)
 {
 	switch (func) {
 	case GGIFUNC_open:
-		*funcptr = GGIopen;
+		*funcptr = (void *)GGIopen;
 		return 0;
 	case GGIFUNC_exit:
 		*funcptr = NULL;
 		return 0;
 	case GGIFUNC_close:
-		*funcptr = GGIclose;
+		*funcptr = (void *)GGIclose;
 		return 0;
 	default:
 		*funcptr = NULL;
