@@ -1,9 +1,10 @@
-/* $Id: libtele.h,v 1.1.1.1 2001/05/12 23:02:27 cegger Exp $
+/* $Id: libtele.h,v 1.5 2005/07/31 15:30:37 soyt Exp $
 ******************************************************************************
 
    libtele.h
 
    Copyright (C) 1998 Andrew Apted	[andrew@ggi-project.org]
+                 2002 Tobias Hunger     [tobias@fresco.org]
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -33,7 +34,7 @@
 
 /*** TYPES ***/
 
-typedef sint32  T_Long;
+typedef int32_t  T_Long;
 
 
 /*** COMMON INFO ***/
@@ -102,6 +103,7 @@ typedef enum tele_event_type
 
 	TELE_CMD_CHECK,		/* check mode */
 	TELE_CMD_OPEN,		/* open window / set mode */
+	TELE_CMD_GETPIXELFMT,   /* request mode info from the client */
 	TELE_CMD_CLOSE,		/* close window / reset mode */
 	TELE_CMD_FLUSH,		/* flush data */
 
@@ -110,6 +112,8 @@ typedef enum tele_event_type
 	TELE_CMD_DRAWBOX,	/* draw a solid box */
 	TELE_CMD_COPYBOX,	/* copy a box elsewhere */
 	TELE_CMD_PUTSTR,	/* write a string */
+	TELE_CMD_GETCHARSIZE,	/* get char size */
+	TELE_CMD_DRAWLINE,	/* draw a solid line */
 
 	TELE_CMD_SETORIGIN,	/* pan around */
 	TELE_CMD_SETFRAME,	/* change frames */
@@ -164,35 +168,59 @@ typedef struct tele_cmd_open_data
 	T_Long error;		/* replied: error value */
 
 	/* everything below is sent _and_ replied */
-	
+
 	T_Long graphtype;
 	T_Long frames;
-	
-	struct { 
-		T_Long width; 
+
+	struct {
+		T_Long width;
 		T_Long height;
 	} visible;
-		
-	struct { 
-		T_Long width; 
+
+	struct {
+		T_Long width;
 		T_Long height;
 	} virt;
 
-	struct { 
-		T_Long width; 
+	struct {
+		T_Long width;
 		T_Long height;
 	} dot;
 
+	struct {
+		T_Long width;
+		T_Long height;
+	} size;
+
 } TeleCmdOpenData;
 
+typedef struct tele_cmd_pixelfmt_data
+{
+  T_Long depth;
+  T_Long size;
+
+  T_Long red_mask;
+  T_Long green_mask;
+  T_Long blue_mask;
+  T_Long alpha_mask;
+  T_Long clut_mask;
+  T_Long fg_mask;
+  T_Long bg_mask;
+  T_Long texture_mask;
+
+  T_Long flags;
+  T_Long stdformat;
+} TeleCmdPixelFmtData;
+
 typedef struct tele_cmd_getput_data
 {
 	T_Long x;
 	T_Long y;
 	T_Long width;
 	T_Long height;
+	T_Long bpp;		/* used by getbox */
 
-	T_Long pixel[1];    /* raw: (replied): array of width*height */
+	T_Long pixel[1];	/* raw: (replied): array of width*height*bpp */
 
 } TeleCmdGetPutData;
 
@@ -204,7 +232,7 @@ typedef struct tele_cmd_drawbox_data
 	T_Long height;
 
 	T_Long pixel;
-	
+
 } TeleCmdDrawBoxData;
 
 typedef struct tele_cmd_copybox_data
@@ -215,25 +243,43 @@ typedef struct tele_cmd_copybox_data
 	T_Long dy;
 	T_Long width;
 	T_Long height;
-	
+
 } TeleCmdCopyBoxData;
 
 typedef struct tele_cmd_putstr_data
 {
 	T_Long x;
 	T_Long y;
-	T_Long fg;
-	T_Long bg;
-
+        T_Long length;
+        T_Long fg;
+        T_Long bg;
 	T_Long text[1];    /* raw: ASCII string, zero terminated */
-	
+
 } TeleCmdPutStrData;
 
+typedef struct tele_cmd_getcharsize_data
+{
+	T_Long width;
+	T_Long height;
+
+} TeleCmdGetCharSizeData;
+
+typedef struct tele_cmd_drawline_data
+{
+	T_Long x;
+	T_Long y;
+	T_Long xe;
+	T_Long ye;
+
+	T_Long pixel;
+
+} TeleCmdDrawLineData;
+
 typedef struct tele_cmd_setorigin_data
 {
 	T_Long x;
 	T_Long y;
-	
+
 } TeleCmdSetOriginData;
 
 typedef struct tele_cmd_setframe_data
@@ -241,7 +287,7 @@ typedef struct tele_cmd_setframe_data
 	T_Long read;
 	T_Long write;
 	T_Long display;
-	
+
 } TeleCmdSetFrameData;
 
 typedef struct tele_cmd_setpalette_data
@@ -250,7 +296,7 @@ typedef struct tele_cmd_setpalette_data
 	T_Long len;
 
 	T_Long colors[1];    /* RGB 8:8:8 */
-	
+
 } TeleCmdSetPaletteData;
 
 
@@ -262,7 +308,6 @@ typedef struct tele_server
 	int inet;
 	int display;
 	int endianness;
-
 } TeleServer;
 
 typedef struct tele_user
@@ -283,8 +328,8 @@ extern int tserver_read(TeleUser *u, TeleEvent *event);
 extern int tserver_write(TeleUser *u, TeleEvent *event);
 extern int tserver_close(TeleUser *u);
 
-extern void *tserver_new_event(TeleUser *u, TeleEvent *event, 
-			       TeleEventType type, 
+extern void *tserver_new_event(TeleUser *u, TeleEvent *event,
+			       TeleEventType type,
 			       int data_size, int raw_size);
 
 /*** CLIENT INFO ***/
@@ -306,8 +351,8 @@ extern int tclient_poll(TeleClient *c);
 extern int tclient_read(TeleClient *c, TeleEvent *event);
 extern int tclient_write(TeleClient *c, TeleEvent *event);
 
-extern void *tclient_new_event(TeleClient *c, TeleEvent *event, 
-			       TeleEventType type, 
+extern void *tclient_new_event(TeleClient *c, TeleEvent *event,
+			       TeleEventType type,
 			       int data_size, int raw_size);
 
 #endif /* _LIBTELE_H */
