@@ -1,4 +1,4 @@
-/* $Id: mode.c,v 1.3 2002/10/27 18:26:25 skids Exp $
+/* $Id: mode.c,v 1.19 2005/09/19 18:46:44 cegger Exp $
 ******************************************************************************
 
    Tile target: setting modes
@@ -26,31 +26,38 @@
 ******************************************************************************
 */
 
+#include "config.h"
+#include <ggi/display/tile.h>
+#include <ggi/internal/ggi_debug.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-
-#include <ggi/display/tile.h>
+#endif
 
 #include "../common/gt-auto.inc"
 
 int GGI_tile_flush_db(ggi_visual *vis, int x, int y, int w, int h, int tryflag)
 {
-	ggi_tile_priv *priv = LIBGGI_PRIVATE(vis);
+	ggi_tile_priv *priv = TILE_PRIV(vis);
 	int i, width, height;
 	int nx, ny, nw, nh;
 	ggi_visual_t currvis;
 
 #if 0
-	for(i = 0; i<priv->numvis; i++) {
-		currvis = priv->vislist[i];
-		width = priv->vis_sizes[i].x;
-		height = priv->vis_sizes[i].y;
+	DPRINT_MISC("GGI_tile_flush_db(%p, %i, %i, %i, %i, %i) entered\n",
+			vis, x, y, w, h, tryflag);
+
+	for(i = 0; i < priv->numvis; i++) {
+		currvis = priv->vislist[i].vis;
+		width = priv->vislist[i].size.x;
+		height = priv->vislist[i].size.y;
 		
 		ggiGetBox(vis, 
-			priv->vis_origins[i].x+vis->origin_x, 
-			priv->vis_origins[i].y+vis->origin_y,
+			priv->vislist[i].origin.x+vis->origin_x, 
+			priv->vislist[i].origin.y+vis->origin_y,
 			width, height, priv->buf);
 		ggiPutBox(currvis, 0, 0, width, height, priv->buf);
 
@@ -60,23 +67,31 @@ int GGI_tile_flush_db(ggi_visual *vis, int x, int y, int w, int h, int tryflag)
 	*/
 
 	int rowadd, stride;
-	uint8 *buf;
+	uint8_t *buf;
+
+	DPRINT_MISC("GGI_tile_flush_db(%p, %i, %i, %i, %i, %i) entered\n",
+			vis, x, y, w, h, tryflag);
+
+	if (priv->d_frame == NULL) {
+		/* happens, when mansync flushes before a mode is set */
+		return GGI_OK;
+	}
 		
 	if(priv->use_db) {
 		MANSYNC_ignore(vis);
 	}
-                        
-	rowadd = (LIBGGI_PIXFMT(vis)->size+7)/8/sizeof(uint8);
-	stride = priv->d_frame->buffer.plb.stride;
-	
-	for(i = 0; i<priv->numvis; i++) {
-		currvis = priv->vislist[i];
-		width = priv->vis_sizes[i].x;
-		height = priv->vis_sizes[i].y - 1;
 
-		buf = (uint8*)priv->d_frame->read +
-				stride * (priv->vis_origins[i].y + vis->origin_y + height) +
-				rowadd * (priv->vis_origins[i].x + vis->origin_x);
+	rowadd = (LIBGGI_PIXFMT(vis)->size+7)/8/sizeof(uint8_t);
+	stride = priv->d_frame->buffer.plb.stride;
+
+	for(i = 0; i<priv->numvis; i++) {
+		currvis = priv->vislist[i].vis;
+		width = priv->vislist[i].size.x;
+		height = priv->vislist[i].size.y - 1;
+
+		buf = (uint8_t*)priv->d_frame->read +
+				stride * (priv->vislist[i].origin.y + vis->origin_y + height) +
+				rowadd * (priv->vislist[i].origin.x + vis->origin_x);
 
 		do {
 			ggiPutHLine(currvis, 0, height, width, buf);
@@ -84,10 +99,10 @@ int GGI_tile_flush_db(ggi_visual *vis, int x, int y, int w, int h, int tryflag)
 		} while(height--);
 #endif
 
-		nx = x - priv->vis_origins[i].x;
-		nw = w - priv->vis_origins[i].x;
-		ny = y - priv->vis_origins[i].y;
-		nh = h - priv->vis_origins[i].y;
+		nx = x - priv->vislist[i].origin.x;
+		nw = w - priv->vislist[i].origin.x;
+		ny = y - priv->vislist[i].origin.y;
+		nh = h - priv->vislist[i].origin.y;
 		if (nx < 0) nx = 0;
 		else if (nx > LIBGGI_X(currvis)) continue;
 		if (ny < 0) ny = 0;
@@ -102,58 +117,57 @@ int GGI_tile_flush_db(ggi_visual *vis, int x, int y, int w, int h, int tryflag)
 		MANSYNC_cont(vis);
 	}
 
+	DPRINT_MISC("GGI_tile_flush_db: leaving\n");
+
 	return 0;
 }
 
 int GGI_tile_getapi(ggi_visual *vis, int num, char *apiname, char *arguments)
 {
+	*arguments = '\0';
 	switch(num) {
 		case 0:
 			strcpy(apiname, "display-tile");
-			strcpy(arguments, "");
 			return 0;
 		case 1:
 			strcpy(apiname, "generic-stubs");
-			strcpy(arguments, "");
 			return 0;
 		case 2:
 			if(!TILE_PRIV(vis)->use_db)
-				return -1;
-
-			strcpy(arguments, "");
+				return GGI_ENOMATCH;
 
 			if (GT_SCHEME(LIBGGI_GT(vis)) == GT_TEXT) {
-				sprintf(apiname, "generic-text-%d",
+				sprintf(apiname, "generic-text-%u",
 					GT_SIZE(LIBGGI_GT(vis))); 
 				return 0;
 			}
 
-			sprintf(apiname, "generic-linear-%d%s", 
+			sprintf(apiname, "generic-linear-%u%s", 
 				GT_SIZE(LIBGGI_GT(vis)),
 				(LIBGGI_GT(vis) & GT_SUB_HIGHBIT_RIGHT) 
 				? "-r" : "");
 			return 0;
 	}
 
-	return -1;
+	return GGI_ENOMATCH;
 }
 
 
 static int _GGIdomode(ggi_visual *vis)
 {
 	int err, id;
-	char sugname[256], args[256];
+	char sugname[GGI_MAX_APILEN], args[GGI_MAX_APILEN];
 
 	_ggiZapMode(vis, 0);
 
 	for (id=1; 0==GGI_tile_getapi(vis, id, sugname, args); id++) {
-		err = _ggiOpenDL(vis, sugname, args, NULL);
+		err = _ggiOpenDL(vis, _ggiGetConfigHandle(), sugname, args, NULL);
 		if (err) {
 			fprintf(stderr, "display-tile: Can't open the %s (%s) library.\n",
 				sugname, args);
 			return GGI_EFATAL;
 		} else {
-			GGIDPRINT("Success in loading %s (%s)\n", sugname, args);
+			DPRINT("Success in loading %s (%s)\n", sugname, args);
 		}
 	}
 
@@ -202,7 +216,7 @@ static int _GGIdomode(ggi_visual *vis)
 	vis->opcolor->unmappixel=GGI_tile_unmappixel;
 	vis->opcolor->setpalvec=GGI_tile_setpalvec;
 	vis->opcolor->getpalvec=GGI_tile_getpalvec;
-
+	
 	ggiIndicateChange(vis, GGI_CHG_APILIST);
 
 	return 0;
@@ -216,10 +230,12 @@ int GGI_tile_setmode(ggi_visual *vis,ggi_mode *tm)
 	ggi_mode sugmode;
 	int depth, err, i;
 
+	DPRINT_MODE("GGI_tile_setmode(%p, %p) entered\n",
+			(void *)vis, (void *)tm);
 	err = GGI_tile_checkmode(vis, tm);
 	if (err) return err;
 	
-	priv=LIBGGI_PRIVATE(vis);
+	priv = TILE_PRIV(vis);
 
 	depth = GT_SIZE(tm->graphtype);
 
@@ -246,22 +262,22 @@ int GGI_tile_setmode(ggi_visual *vis,ggi_mode *tm)
 			LIBGGI_APPBUFS(vis)[i]->read = LIBGGI_APPBUFS(vis)[i]->write = fbaddr;
 			LIBGGI_APPBUFS(vis)[i]->layout = blPixelLinearBuffer;
 			LIBGGI_APPBUFS(vis)[i]->buffer.plb.stride
-				= ((GT_SIZE(tm->graphtype) * tm->virt.x)+7)/8;
+				= GT_ByPPP(tm->virt.x, tm->graphtype);
 		}
 	}
 	
-	for(i = 0; i<priv->numvis; i++) {
-		currvis = priv->vislist[i];
+	for (i = 0; i<priv->numvis; i++) {
+		currvis = priv->vislist[i].vis;
 		sugmode = *tm;
-		sugmode.visible.x = priv->vis_sizes[i].x;
-		sugmode.visible.y = priv->vis_sizes[i].y;
+		sugmode.visible.x = priv->vislist[i].size.x;
+		sugmode.visible.y = priv->vislist[i].size.y;
 		sugmode.virt.x=sugmode.virt.y=GGI_AUTO;
 
 		/* Multiple buffering is handled by us in DB mode */
 		if(priv->use_db)
 			sugmode.frames = 1;
 
-		GGIDPRINT("Setting mode for visual #%d...\n", i);
+		DPRINT("Setting mode for visual #%d...\n", i);
 
 		/* Set mode mantra from lib/libggi/mode.c.  Be careful here.
 		   See GGIcheckmode() for why we do this. */
@@ -272,23 +288,23 @@ int GGI_tile_setmode(ggi_visual *vis,ggi_mode *tm)
 			return err;
 		}
 
-		GGIDPRINT("Success setting mode for visual #%d\n", i);
+		DPRINT("Success setting mode for visual #%d\n", i);
 
 		if(!priv->use_db) {
 			/* Adjust clipping rectangle for mode dimensions. */
-		
-			priv->vis_clipbr[i].x = priv->vis_origins[i].x + priv->vis_sizes[i].x;
-			if(priv->vis_clipbr[i].x > tm->virt.x)
-				priv->vis_clipbr[i].x = tm->virt.x;
 
-			priv->vis_clipbr[i].y = priv->vis_origins[i].y + priv->vis_sizes[i].y;
-			if(priv->vis_clipbr[i].y > tm->virt.y)
-				priv->vis_clipbr[i].y = tm->virt.y;
+			priv->vislist[i].clipbr.x = priv->vislist[i].origin.x + priv->vislist[i].size.x;
+			if(priv->vislist[i].clipbr.x > tm->virt.x)
+				priv->vislist[i].clipbr.x = tm->virt.x;
+
+			priv->vislist[i].clipbr.y = priv->vislist[i].origin.y + priv->vislist[i].size.y;
+			if(priv->vislist[i].clipbr.y > tm->virt.y)
+				priv->vislist[i].clipbr.y = tm->virt.y;
 		}
 
 #if 0
 		/* This is to determine the largest buffer size needed for copybox */
-		currbuf = priv->vis_sizes[i].x * priv->vis_sizes[i].y;
+		currbuf = priv->vislist[i].size.x * priv->vislist[i].size.y;
 		if(currbuf > maxbuf) maxbuf=currbuf;
 	}
 
@@ -305,23 +321,27 @@ int GGI_tile_setmode(ggi_visual *vis,ggi_mode *tm)
 	}
 
 	/* Assume first visual's pixelformat properties */
-	memcpy(LIBGGI_PIXFMT(vis), LIBGGI_PIXFMT(priv->vislist[0]), 
+	memcpy(LIBGGI_PIXFMT(vis), LIBGGI_PIXFMT(priv->vislist[0].vis), 
 		sizeof(ggi_pixelformat));
 
 	memcpy(LIBGGI_MODE(vis),tm,sizeof(ggi_mode));
 
-	if(_GGIdomode(vis))
-		return -1;
+	err = _GGIdomode(vis);
+	if (err != 0) return err;
 
-	if(priv->use_db) {
-		for(i = 0; i<tm->frames; i++)
+	if (priv->use_db) {
+		DPRINT_MISC("GGI_tile_setmode: setting up directbuffer\n");
+		for (i = 0; i<tm->frames; i++)
 			LIBGGI_APPBUFS(vis)[i]->buffer.plb.pixelformat = LIBGGI_PIXFMT(vis);
 
 		priv->d_frame = LIBGGI_APPBUFS(vis)[0];
 
+		DPRINT_MISC("GGI_tile_setmode: call MANSYNC_SETFLAGS");
+		MANSYNC_SETFLAGS(vis, LIBGGI_FLAGS(vis));
 		MANSYNC_cont(vis);
 	}
-	
+
+	DPRINT_MISC("GGI_tile_setmode: leaving\n");	
 	return 0;
 }
 
@@ -330,17 +350,17 @@ int GGI_tile_setmode(ggi_visual *vis,ggi_mode *tm)
 /**********************************/
 int GGI_tile_checkmode(ggi_visual *vis,ggi_mode *tm)
 {
-	ggi_tile_priv *priv = LIBGGI_PRIVATE(vis);
+	ggi_tile_priv *priv = TILE_PRIV(vis);
 	ggi_mode sugmode;
 	int err, i;
 
 	/* Find out the "bounding rectangle" for GGI_AUTO values */
-	if (tm->virt.x==GGI_AUTO) {
+	if(tm->virt.x==GGI_AUTO) {
 		int x;
 		tm->virt.x=0;
-		for (i = 0; i<priv->numvis; i++) {
-			x = priv->vis_origins[i].x
-				+ priv->vis_sizes[i].x;
+		for(i = 0; i<priv->numvis; i++) {
+			x = priv->vislist[i].origin.x
+				+ priv->vislist[i].size.x;
 			if (x > tm->virt.x) tm->virt.x = x;
 		}
 	}
@@ -348,8 +368,8 @@ int GGI_tile_checkmode(ggi_visual *vis,ggi_mode *tm)
 		int y;
 		tm->virt.y=0;
 		for(i = 0; i<priv->numvis; i++) {
-			y = priv->vis_origins[i].y
-				+ priv->vis_sizes[i].y;
+			y = priv->vislist[i].origin.y
+				+ priv->vislist[i].size.y;
 			if (y > tm->virt.y) tm->virt.y = y;
 		}
 	}
@@ -373,15 +393,15 @@ int GGI_tile_checkmode(ggi_visual *vis,ggi_mode *tm)
 		   GGI_AUTO to be substituted by GGI_DEFMODE values */
 		   
 		sugmode.frames = priv->use_db ? 1 : tm->frames;
-		sugmode.visible.x = priv->vis_sizes[i].x;
-		sugmode.visible.y = priv->vis_sizes[i].y;
+		sugmode.visible.x = priv->vislist[i].size.x;
+		sugmode.visible.y = priv->vislist[i].size.y;
 		sugmode.virt.x = sugmode.virt.y = GGI_AUTO;
 		sugmode.size.x = sugmode.size.y = GGI_AUTO;
 		sugmode.graphtype = tm->graphtype;
 		sugmode.dpp  = tm->dpp;
 		sugmode.size = tm->size;
 
-		err = ggiCheckMode(priv->vislist[i], &sugmode);
+		err = ggiCheckMode(priv->vislist[i].vis, &sugmode);
 		if (err) {
 			/* Forget searching all visuals for the source of
 			   error, it's way too complicated. Just say fail
@@ -420,18 +440,18 @@ int GGI_tile_setflags(ggi_visual *vis,ggi_flags flags)
 	ggi_tile_priv *priv = TILE_PRIV(vis);
 
 	LIBGGI_FLAGS(vis) = flags;
+	/* Unkown flags don't take. */
+	LIBGGI_FLAGS(vis) &= GGIFLAG_ASYNC;
 
 	if (priv->use_db) {
 		MANSYNC_SETFLAGS(vis, flags);
 	} else {
 		int i;
 		for (i = 0; i<priv->numvis; i++) {
-			ggiSetFlags(priv->vislist[i], flags);
+			ggiSetFlags(priv->vislist[i].vis, flags);
 		}
 	}	
 
-	LIBGGI_FLAGS(vis) &= GGIFLAG_ASYNC; /* Unkown flags don't take. */
-
 	return 0;
 }
 
@@ -441,7 +461,7 @@ int GGI_tile_setorigin(ggi_visual *vis,int x,int y)
 
 	if ( x<0 || x> mode->virt.x-mode->visible.x ||
 	     y<0 || y> mode->virt.y-mode->visible.y )
-	     return -1;
+	     return GGI_ENOSPACE;
 
 	vis->origin_x=x;
 	vis->origin_y=y;
