@@ -1,4 +1,4 @@
-/* $Id: stubs.c,v 1.1.1.1 2001/05/12 23:02:36 cegger Exp $
+/* $Id: stubs.c,v 1.7 2005/07/30 10:58:28 cegger Exp $
 ******************************************************************************
 
    Code stolen from the graphics library for GGI.
@@ -31,6 +31,7 @@
 ******************************************************************************
 */
 
+#include "config.h"
 #include <stdlib.h>
 #include <ggi/display/tile.h>
 
@@ -40,7 +41,7 @@ int GGI_tile_flush(ggi_visual *vis, int x, int y, int w, int h, int tryflag)
 	int i;
 
 	for (i=0; i < priv->numvis; i++) {
-		ggiFlushRegion(priv->vislist[i], x, y, w, h);
+		ggiFlushRegion(priv->vislist[i].vis, x, y, w, h);
 	}
 
 	return 0;
@@ -60,7 +61,7 @@ void GGI_tile_gcchanged(ggi_visual *vis, int mask)
 		mask &= ~GGI_GCCHANGED_CLIP;
 
 	for(i=0; i<priv->numvis; i++) {
-		currvis = priv->vislist[i];
+		currvis = priv->vislist[i].vis;
 
 #if 0	/* Don't blindly copy the GC. */
 		memcpy(LIBGGI_GC(currvis), LIBGGI_GC(vis), sizeof(ggi_gc));
@@ -91,8 +92,8 @@ int GGI_tile_drawbox(ggi_visual *vis, int _x, int _y, int _width, int _length)
 	int i, x, y, width, length, diff;
 
 	for(i=0; i<priv->numvis; i++) {
-		cliptl = priv->vis_origins[i];
-		clipbr = priv->vis_clipbr[i];
+		cliptl = priv->vislist[i].origin;
+		clipbr = priv->vislist[i].clipbr;
 		x = _x;
 		y = _y;
 		width = _width;
@@ -119,14 +120,14 @@ int GGI_tile_drawbox(ggi_visual *vis, int _x, int _y, int _width, int _length)
 		if (length <= 0 || width <= 0 )
 			continue;
 
-		ggiDrawBox(priv->vislist[i],
+		ggiDrawBox(priv->vislist[i].vis,
 			x - cliptl.x, y - cliptl.y, width, length);
 	}
 
 	return 0;
 }
 
-int GGI_tile_putbox(ggi_visual *vis, int _x, int _y, int _width, int _length, void *buffer)
+int GGI_tile_putbox(ggi_visual *vis, int _x, int _y, int _width, int _length, const void *buffer)
 {
 	ggi_tile_priv *priv = TILE_PRIV(vis);
 	int rowadd = (LIBGGI_PIXFMT(vis)->size+7)/8;
@@ -134,8 +135,8 @@ int GGI_tile_putbox(ggi_visual *vis, int _x, int _y, int _width, int _length, vo
 	int i, x, y, width, length, diff;
 
 	for(i=0; i<priv->numvis; i++) {
-		cliptl = priv->vis_origins[i];
-		clipbr = priv->vis_clipbr[i];
+		cliptl = priv->vislist[i].origin;
+		clipbr = priv->vislist[i].clipbr;
 		x = _x;
 		y = _y;
 		width = _width;
@@ -163,9 +164,9 @@ int GGI_tile_putbox(ggi_visual *vis, int _x, int _y, int _width, int _length, vo
 			continue;
 
 		while(length--) {
-			ggiPutHLine(priv->vislist[i],
+			ggiPutHLine(priv->vislist[i].vis,
 				x - cliptl.x, y - cliptl.y + length, width,
-				((uint8*)buffer + rowadd*_width*(y-_y+length) + rowadd*(x-_x)));
+				((const uint8_t*)buffer + rowadd*_width*(y-_y+length) + rowadd*(x-_x)));
 		}
 	}
 
@@ -180,8 +181,8 @@ int GGI_tile_getbox(ggi_visual *vis, int _x, int _y, int _width, int _length, vo
 	int i, x, y, width, length, diff;
 
 	for(i=0; i<priv->numvis; i++) {
-		cliptl = priv->vis_origins[i];
-		clipbr = priv->vis_clipbr[i];
+		cliptl = priv->vislist[i].origin;
+		clipbr = priv->vislist[i].clipbr;
 		x = _x;
 		y = _y;
 		width = _width;
@@ -209,9 +210,9 @@ int GGI_tile_getbox(ggi_visual *vis, int _x, int _y, int _width, int _length, vo
 			continue;
 
 		while(length--) {
-			ggiGetHLine(priv->vislist[i],
+			ggiGetHLine(priv->vislist[i].vis,
 				x - cliptl.x, y - cliptl.y + length, width,
-				((uint8*)buffer + rowadd*_width*(y-_y+length) + rowadd*(x-_x)));
+				((uint8_t*)buffer + rowadd*_width*(y-_y+length) + rowadd*(x-_x)));
 		}
 	}
 
@@ -230,8 +231,8 @@ int GGI_tile_copybox(ggi_visual *vis, int x, int y, int width, int height,
 	   contained in one of the tile visuals.
 	*/
 	for (i=0; i < priv->numvis; i++) {
-		cliptl = priv->vis_origins[i];
-		clipbr = priv->vis_clipbr[i];
+		cliptl = priv->vislist[i].origin;
+		clipbr = priv->vislist[i].clipbr;
 
 		if (x < cliptl.x || y < cliptl.y ||
 		    x + width > clipbr.x || y + height > clipbr.y ||
@@ -240,13 +241,13 @@ int GGI_tile_copybox(ggi_visual *vis, int x, int y, int width, int height,
 			continue;
 		}
 
-		return ggiCopyBox(priv->vislist[i],
+		return ggiCopyBox(priv->vislist[i].vis,
 				  x - cliptl.x, y - cliptl.y, width, height,
 				  nx - cliptl.x, ny - cliptl.y);
 	}
 
 	/* ARGGH!!! */
-	buf = malloc((LIBGGI_PIXFMT(vis)->size+7)/8*width*height);
+	buf = malloc((size_t)((LIBGGI_PIXFMT(vis)->size+7)/8*width*height));
 	if (buf == NULL) {
 		/* Tough luck kid... */
 		return GGI_ENOMEM;
@@ -266,7 +267,7 @@ int GGI_tile_fillscreen(ggi_visual *vis)
 	int i;
 
 	for(i = 0; i<priv->numvis; i++)
-		ggiFillscreen(priv->vislist[i]);
+		ggiFillscreen(priv->vislist[i].vis);
 
 	return 0;
 }
@@ -275,8 +276,11 @@ int GGI_tile_fillscreen(ggi_visual *vis)
 int GGI_tile_putc(ggi_visual *vis,int x,int y,char c)
 {
 	int err=EOK;
-	ggi_visual_t *vislist=TILE_PRIV(vis);
-	while(*vislist) err=ggiPutc(*vislist++,x,y,c);
+	int i;
+	ggi_tile_priv *priv = TILE_PRIV(vis);
+
+	for (i = 0; i < priv->numvis; i++)
+		err = ggiPutc(priv->vislist[i].vis,x,y,c);
 
 	return err;
 }
@@ -285,8 +289,11 @@ int GGI_tile_putc(ggi_visual *vis,int x,int y,char c)
 int GGI_tile_puts(ggi_visual *vis,int x,int y,const char *str)
 {
 	int err=EOK;
-	ggi_visual_t *vislist=TILE_PRIV(vis);
-	while(*vislist) err=ggiPuts(*vislist++,x,y,str);
+	int i;
+	ggi_tile_priv *priv = TILE_PRIV(vis);
+
+	for (i = 0; i < priv->numvis; i++)
+		err = ggiPuts(priv->vislist[i],x,y,str);
 
 	return err;
 }
@@ -294,8 +301,11 @@ int GGI_tile_puts(ggi_visual *vis,int x,int y,const char *str)
 int GGI_tile_drawline(ggi_visual *vis,int x1,int y1,int x2,int y2)
 {
 	int err=EOK;
-	ggi_visual_t *vislist=TILE_PRIV(vis);
-	while(*vislist) err=ggiDrawLine(*vislist++,x1,y1,x2,y2);
+	int i;
+	ggi_tile_priv *priv = TILE_PRIV(vis);
+
+	for (i = 0; i < priv->numvis; i++)
+		err = ggiDrawLine(priv->vislist[i],x1,y1,x2,y2);
 
 	return err;
 }
@@ -308,8 +318,8 @@ int GGI_tile_drawhline_nc(ggi_visual *vis,int _x,int y,int _width)
 	int i, x, width, diff;
 
 	for(i=0; i<priv->numvis; i++) {
-		cliptl = priv->vis_origins[i];
-		clipbr = priv->vis_clipbr[i];
+		cliptl = priv->vislist[i].origin;
+		clipbr = priv->vislist[i].clipbr;
 		x = _x;
 		width = _width;
 
@@ -328,7 +338,7 @@ int GGI_tile_drawhline_nc(ggi_visual *vis,int _x,int y,int _width)
 		if (width <= 0)
 			continue;
 
-		_ggiDrawHLineNC(priv->vislist[i],
+		_ggiDrawHLineNC(priv->vislist[i].vis,
 			x - cliptl.x, y - cliptl.y, width);
 	}
 
@@ -353,7 +363,7 @@ int GGI_tile_drawhline(ggi_visual *vis,int x,int y,int w)
 		return 0;	/* ??? Shouldn't this be an error? */
 }
 
-int GGI_tile_puthline(ggi_visual *vis,int _x,int y,int _width,void *buffer)
+int GGI_tile_puthline(ggi_visual *vis,int _x,int y,int _width,const void *buffer)
 {
 	ggi_tile_priv *priv = TILE_PRIV(vis);
 	int rowadd = (LIBGGI_PIXFMT(vis)->size+7)/8;
@@ -361,8 +371,8 @@ int GGI_tile_puthline(ggi_visual *vis,int _x,int y,int _width,void *buffer)
 	int i, x, width, diff;
 
 	for(i=0; i<priv->numvis; i++) {
-		cliptl = priv->vis_origins[i];
-		clipbr = priv->vis_clipbr[i];
+		cliptl = priv->vislist[i].origin;
+		clipbr = priv->vislist[i].clipbr;
 		x = _x;
 		width = _width;
 
@@ -383,9 +393,9 @@ int GGI_tile_puthline(ggi_visual *vis,int _x,int y,int _width,void *buffer)
 		if (width <= 0)
 			continue;
 
-		ggiPutHLine(priv->vislist[i],
+		ggiPutHLine(priv->vislist[i].vis,
 			x - cliptl.x, y - cliptl.y, width,
-			((uint8*)buffer + diff*rowadd));
+			((const uint8_t*)buffer + diff*rowadd));
 	}
 
 	return 0;
@@ -399,8 +409,8 @@ int GGI_tile_gethline(ggi_visual *vis,int _x,int y,int _width,void *buffer)
 	int i, x, width, diff;
 
 	for(i=0; i<priv->numvis; i++) {
-		cliptl = priv->vis_origins[i];
-		clipbr = priv->vis_clipbr[i];
+		cliptl = priv->vislist[i].origin;
+		clipbr = priv->vislist[i].clipbr;
 		x = _x;
 		width = _width;
 
@@ -421,9 +431,9 @@ int GGI_tile_gethline(ggi_visual *vis,int _x,int y,int _width,void *buffer)
 		if (width <= 0)
 			continue;
 
-		ggiGetHLine(priv->vislist[i],
+		ggiGetHLine(priv->vislist[i].vis,
 			x - cliptl.x, y - cliptl.y, width,
-			((uint8*)buffer + diff*rowadd));
+			((uint8_t*)buffer + diff*rowadd));
 	}
 
 	return 0;
@@ -436,8 +446,8 @@ int GGI_tile_drawvline_nc(ggi_visual *vis,int x,int _y,int _height)
 	int i, y, length, diff;
 
 	for(i=0; i<priv->numvis; i++) {
-		cliptl = priv->vis_origins[i];
-		clipbr = priv->vis_clipbr[i];
+		cliptl = priv->vislist[i].origin;
+		clipbr = priv->vislist[i].clipbr;
 		y = _y;
 		length = _height;
 
@@ -456,7 +466,7 @@ int GGI_tile_drawvline_nc(ggi_visual *vis,int x,int _y,int _height)
 		if (length <= 0)
 			continue;
 
-		_ggiDrawVLineNC(priv->vislist[i],
+		_ggiDrawVLineNC(priv->vislist[i].vis,
 			x - cliptl.x, y - cliptl.y, length);
  	}
 
@@ -483,7 +493,7 @@ int GGI_tile_drawvline(ggi_visual *vis,int x,int y,int height)
 		return 0;	/* ??? Shouldn't this be an error? */
 }
 
-int GGI_tile_putvline(ggi_visual *vis,int x,int _y,int _height,void *buffer)
+int GGI_tile_putvline(ggi_visual *vis,int x,int _y,int _height,const void *buffer)
 {
 	ggi_tile_priv *priv = TILE_PRIV(vis);
 	int rowadd = (LIBGGI_PIXFMT(vis)->size+7)/8;
@@ -491,8 +501,8 @@ int GGI_tile_putvline(ggi_visual *vis,int x,int _y,int _height,void *buffer)
 	int i, y, length, diff;
 
 	for(i=0; i<priv->numvis; i++) {
-		cliptl = priv->vis_origins[i];
-		clipbr = priv->vis_clipbr[i];
+		cliptl = priv->vislist[i].origin;
+		clipbr = priv->vislist[i].clipbr;
 		y = _y;
 		length = _height;
 
@@ -513,9 +523,9 @@ int GGI_tile_putvline(ggi_visual *vis,int x,int _y,int _height,void *buffer)
 		if (length <= 0)
 			continue;
 
-		ggiPutVLine(priv->vislist[i],
+		ggiPutVLine(priv->vislist[i].vis,
 			x - cliptl.x, y - cliptl.y, length,
-			((uint8*)buffer + diff*rowadd));
+			((const uint8_t*)buffer + diff*rowadd));
  	}
 
 	return 0;
@@ -529,8 +539,8 @@ int GGI_tile_getvline(ggi_visual *vis,int x,int _y,int _height,void *buffer)
 	int i, y, length, diff;
 
 	for(i=0; i<priv->numvis; i++) {
-		cliptl = priv->vis_origins[i];
-		clipbr = priv->vis_clipbr[i];
+		cliptl = priv->vislist[i].origin;
+		clipbr = priv->vislist[i].clipbr;
 		y = _y;
 		length = _height;
 
@@ -551,9 +561,9 @@ int GGI_tile_getvline(ggi_visual *vis,int x,int _y,int _height,void *buffer)
 		if (length <= 0)
 			continue;
 
-		ggiGetVLine(priv->vislist[i],
+		ggiGetVLine(priv->vislist[i].vis,
 			x - cliptl.x, y - cliptl.y, length,
-			((uint8*)buffer + diff*rowadd));
+			((uint8_t*)buffer + diff*rowadd));
  	}
 
 	return 0;
@@ -566,15 +576,15 @@ int GGI_tile_drawpixel_nc(ggi_visual *vis,int x,int y)
 	int i;
 
 	for(i=0; i<priv->numvis; i++) {
-		cliptl = priv->vis_origins[i];
-		clipbr = priv->vis_clipbr[i];
+		cliptl = priv->vislist[i].origin;
+		clipbr = priv->vislist[i].clipbr;
 
 		if (x < cliptl.x || y < cliptl.y ||
 			x >= clipbr.x || y >= clipbr.y)
 			continue;
 
 		/* Do we disallow overlapping tiles? */
-		_ggiDrawPixelNC(priv->vislist[i],
+		_ggiDrawPixelNC(priv->vislist[i].vis,
 			x - cliptl.x, y - cliptl.y);
 	}
 
@@ -594,15 +604,15 @@ int GGI_tile_putpixel_nc(ggi_visual *vis, int x, int y, ggi_pixel col)
 	int i;
 
 	for(i=0; i<priv->numvis; i++) {
-		cliptl = priv->vis_origins[i];
-		clipbr = priv->vis_clipbr[i];
+		cliptl = priv->vislist[i].origin;
+		clipbr = priv->vislist[i].clipbr;
 
 		if (x < cliptl.x || y < cliptl.y ||
 			x >= clipbr.x || y >= clipbr.y)
 			continue;
 
 		/* Do we disallow overlapping tiles? */
-		ggiPutPixel(priv->vislist[i],
+		ggiPutPixel(priv->vislist[i].vis,
 			x - cliptl.x, y - cliptl.y, col);
 	}
 
@@ -622,16 +632,16 @@ int GGI_tile_getpixel(ggi_visual *vis, int x, int y, ggi_pixel *col)
 	int i;
 
 	for(i=0; i<priv->numvis; i++) {
-		cliptl = priv->vis_origins[i];
-		clipbr = priv->vis_clipbr[i];
+		cliptl = priv->vislist[i].origin;
+		clipbr = priv->vislist[i].clipbr;
 
 		if (x < cliptl.x || y < cliptl.y ||
 			x >= clipbr.x || y >= clipbr.y)
 			continue;
 
-		return ggiGetPixel(priv->vislist[i],
+		return ggiGetPixel(priv->vislist[i].vis,
 			x - cliptl.x, y - cliptl.y, col);
 	}
 
-	return -1;
+	return GGI_ENOSPACE;
 }
