@@ -1,4 +1,4 @@
-/* $Id: hline.c,v 1.1.1.1 2001/05/12 23:02:42 cegger Exp $
+/* $Id: hline.c,v 1.4 2005/07/30 11:38:51 cegger Exp $
 ******************************************************************************
 
    SVGAlib target vgagl helper: horizontal lines
@@ -25,6 +25,7 @@
 ******************************************************************************
 */
 
+#include "config.h"
 #include <ggi/internal/ggi-dl.h>
 #include "vgaglvis.h"
 
@@ -45,10 +46,10 @@ int GGI_vgagl_drawhline_nc(ggi_visual *vis, int x, int y, int w)
 	return 0;
 }
 
-int GGI_vgagl_puthline(ggi_visual *vis, int x, int y, int w, void *buffer)
+int GGI_vgagl_puthline(ggi_visual *vis, int x, int y, int w, const void *buffer)
 { 
 	int pixelsize = (LIBGGI_PIXFMT(vis)->size+7)/8;
-	uint8 *buf = buffer;
+	const uint8_t *buf = buffer;
 
 	LIBGGICLIP_XYW_BUFMOD(vis, x, y, w, buf, *pixelsize);
 
