@@ -1,4 +1,4 @@
-/* $Id: visual.c,v 1.2 2002/09/08 21:37:47 soyt Exp $
+/* $Id: visual.c,v 1.9 2005/07/30 11:38:52 cegger Exp $
 ******************************************************************************
 
    FreeBSD vgl(3) target: initialization
@@ -26,7 +26,6 @@
 */
 
 #include "config.h"
-#include <ggi/internal/ggi-dl.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,10 +33,16 @@
 #include <errno.h>
 
 #include <ggi/display/vgl.h>
+#include <ggi/internal/ggi_debug.h>
 
 static int usagecounter = 0;
 
-void _GGI_vgl_freedbs(ggi_visual *vis) {
+
+
+void _GGI_vgl_freedbs(ggi_visual *vis);
+
+void _GGI_vgl_freedbs(ggi_visual *vis)
+{
 	int i;
 
 	for (i=LIBGGI_APPLIST(vis)->num-1; i >= 0; i--) {
@@ -53,7 +58,7 @@ static int _GGIcheckvglmodes(ggi_visual *vis)
 	int i;
 	vgl_priv *priv = VGL_PRIV(vis);
 
-	GGIDPRINT_MISC("display-vgl: Checking modes\n");
+	DPRINT_MISC("display-vgl: Checking modes\n");
 
 	for (i=1; i <= M_VESA_MODE_MAX; i++) {
 		modeinfo.vi_mode = i;
@@ -89,14 +94,14 @@ static int _GGIcheckvglmodes(ggi_visual *vis)
 			sgmode->gt = GT_CONSTRUCT(bpp, (bpp <= 8) ?
 			      GT_PALETTE : GT_TRUECOLOR, size);
 
-			GGIDPRINT_MISC("display-vgl: found mode %dx%dx%d %d\n",
+			DPRINT_MISC("display-vgl: found mode %dx%dx%d %d\n",
 				sgmode->x, sgmode->y, sgmode->bpp, size);
 		}
 	}
 
 	if (modes == 0) {
-		GGIDPRINT_MISC("display-vgl: no mode!\n");
-		return -1;
+		DPRINT_MISC("display-vgl: no mode!\n");
+		return GGI_ENOMATCH;
 	} else {
 		priv->availmodes = realloc(priv->availmodes,
 					   (modes+1)*sizeof(ggi_modelistmode));
@@ -109,7 +114,7 @@ static int _GGIcheckvglmodes(ggi_visual *vis)
 static int 
 GGI_vgl_sendevent(ggi_visual *vis, gii_event *ev)
 {
-	GGIDPRINT_MISC("GGI_vgl_sendevent() called\n");
+	DPRINT_MISC("GGI_vgl_sendevent() called\n");
 
 	if (ev->any.type != evCommand) {
 		return GGI_EEVUNKNOWN;
@@ -118,7 +123,7 @@ GGI_vgl_sendevent(ggi_visual *vis, gii_event *ev)
 #ifdef notyet
 	switch (ev->cmd.code) {
 	case GGICMD_ACKNOWLEDGE_SWITCH:
-		GGIDPRINT_MISC("display-vgl: switch acknowledge\n");
+		DPRINT_MISC("display-vgl: switch acknowledge\n");
 		if (priv->switchpending) {
 			priv->doswitch(vis);
 			return 0;
@@ -128,12 +133,12 @@ GGI_vgl_sendevent(ggi_visual *vis, gii_event *ev)
 		}
 		break;
 	case GGICMD_NOHALT_ON_UNMAP:
-		GGIDPRINT_MISC("display-vgl: nohalt on\n");
+		DPRINT_MISC("display-vgl: nohalt on\n");
 		priv->dohalt = 0;
 		priv->autoswitch = 0;
 		break;
 	case GGICMD_HALT_ON_UNMAP:
-		GGIDPRINT_MISC("display-vgl: halt on\n");
+		DPRINT_MISC("display-vgl: halt on\n");
 		priv->dohalt = 1;
 		priv->autoswitch = 1;
 		if (priv->switchpending) {
@@ -180,7 +185,7 @@ static int do_cleanup(ggi_visual *vis)
 
 
 static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
-			const char *args, void *argptr, uint32 *dlret)
+			const char *args, void *argptr, uint32_t *dlret)
 {
 	vgl_priv *priv;
 	int error;
@@ -225,12 +230,12 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 
 	if (args) {
 		if (strncmp(args, "-usedb:", 7) == 0) {
-			GGIDPRINT_MISC("display-vgl: Enabling DB\n");
+			DPRINT_MISC("display-vgl: Enabling DB\n");
 			priv->vgl_use_db = 1;
 			args += 7;
 		}
 		if (strncmp(args, "-nodb:", 6) == 0) {
-			GGIDPRINT_MISC("display-vgl: Disabling DB\n");
+			DPRINT_MISC("display-vgl: Disabling DB\n");
 			priv->vgl_use_db = 0;
 			args += 6;
 		}
@@ -294,7 +299,7 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 	return 0;
 
 error:
-	GGIDPRINT("display-vgl: open failed (%d)\n", error);
+	DPRINT("display-vgl: open failed (%d)\n", error);
 	return error;
 }
 
@@ -305,17 +310,20 @@ static int GGIclose(ggi_visual *vis, struct ggi_dlhandle *dlh)
 }
 
 
+EXPORTFUNC
+int GGIdl_vgl(int func, void **funcptr);
+
 int GGIdl_vgl(int func, void **funcptr)
 {
 	switch (func) {
 	case GGIFUNC_open:
-		*funcptr = GGIopen;
+		*funcptr = (void *)GGIopen;
 		return 0;
 	case GGIFUNC_exit:
 		*funcptr = NULL;
 		return 0;
 	case GGIFUNC_close:
-		*funcptr = GGIclose;
+		*funcptr = (void *)GGIclose;
 		return 0;
 	default:
 		*funcptr = NULL;
