@@ -1,4 +1,4 @@
-/* $Id: test2.c,v 1.1.1.1 2001/05/12 23:03:11 cegger Exp $
+/* $Id: test2.c,v 1.5 2005/06/09 18:40:45 cegger Exp $
 ******************************************************************************
 
    Test extension test2.c
@@ -26,11 +26,12 @@
 ******************************************************************************
 */
 
+#include "config.h"
+#include <ggi/internal/internal.h>
+
 #include <stdio.h>
 #include <string.h>
 
-#include <ggi/internal/internal.h>
-
 #include "test1.h"
 #include "test2.h"
 
@@ -45,7 +46,7 @@ ggi_extid ggiTest2ID=-1;
 static int changed(ggi_visual_t vis,int whatchanged)
 {
 	printf("load extension 2 - vis: %p %i\n",
-		vis,whatchanged);
+		(void *)vis, whatchanged);
 	return 0;
 }
 
@@ -74,7 +75,8 @@ int ggiTest2Attach(ggi_visual_t vis)
 {
 	int rc;
 	rc=ggiExtensionAttach(vis,ggiTest2ID);
-	printf("Attached Test2 extension to %p. rc=%i\n", vis, rc);
+	printf("Attached Test2 extension to %p. rc=%i\n",
+		(void *)vis, rc);
 
 	if (rc==0) {	/* We are actually creating the primary instance. */
 		ggiTest1Attach(vis);
@@ -88,7 +90,8 @@ int ggiTest2Detach(ggi_visual_t vis)
 {
 	int rc;
 	rc=ggiExtensionDetach(vis,ggiTest2ID);
-	printf("Detached Test2 extension from %p. rc=%i\n", vis, rc);
+	printf("Detached Test2 extension from %p. rc=%i\n",
+		(void *)vis, rc);
 	if (rc==0) ggiTest1Detach(vis);
 
 	return rc;
@@ -100,7 +103,7 @@ void ggiTest2PrintLocaldata(ggi_visual_t vis)
 	printf("%s\n",(char *)LIBGGI_EXT(vis,ggiTest2ID));
 }
 
-void ggiTest2SetLocaldata  (ggi_visual_t vis,char *content)
+void ggiTest2SetLocaldata  (ggi_visual_t vis,const char *content)
 {
 	strcpy(LIBGGI_EXT(vis,ggiTest2ID),content);
 }
