@@ -1,4 +1,4 @@
-/* $Id: aa.h,v 1.1.1.1 2001/05/12 23:03:19 cegger Exp $
+/* $Id: aa.h,v 1.4 2005/07/31 15:30:38 soyt Exp $
 ******************************************************************************
 
    Headers for AA target.
@@ -34,14 +34,15 @@
 #include <ggi/display/mansync.h>
 
 
-ggifunc_flush		GGI_aa_flush;
+ggifunc_flush			GGI_aa_flush;
 ggifunc_getmode		GGI_aa_getmode;
 ggifunc_setmode		GGI_aa_setmode;
 ggifunc_checkmode	GGI_aa_checkmode;
 ggifunc_getapi 		GGI_aa_getapi;
 ggifunc_setflags	GGI_aa_setflags;
 
-ggifunc_setpalvec	GGI_aa_setpalvec;
+ggifunc_setPalette	GGI_aa_setPalette;
+size_t GGI_aa_getPrivSize(ggi_visual_t);
 
 giifunc_eventpoll	GII_aa_poll;
 /*
@@ -54,18 +55,17 @@ giifunc_sendevent	GII_aa_sendevent;
 #define AA_SCRMULT_X 2
 #define AA_SCRMULT_Y 2
 
-#define MANSYNC_init(vis)   AA_PRIV(vis)->opmansync->init(vis)
-#define MANSYNC_deinit(vis) AA_PRIV(vis)->opmansync->deinit(vis)
-#define MANSYNC_start(vis)  AA_PRIV(vis)->opmansync->start(vis)
-#define MANSYNC_stop(vis)   AA_PRIV(vis)->opmansync->stop(vis)
-#define MANSYNC_ignore(vis) AA_PRIV(vis)->opmansync->ignore(vis)
-#define MANSYNC_cont(vis)   AA_PRIV(vis)->opmansync->cont(vis)
+#define MANSYNC_init(vis)   MANSYNC_DECL_INIT(AA_PRIV(vis), vis)
+#define MANSYNC_deinit(vis) MANSYNC_DECL_DEINIT(AA_PRIV(vis), vis)
+#define MANSYNC_start(vis)  MANSYNC_DECL_START(AA_PRIV(vis), vis)
+#define MANSYNC_stop(vis)   MANSYNC_DECL_STOP(AA_PRIV(vis), vis)
+#define MANSYNC_ignore(vis) MANSYNC_DECL_IGNORE(AA_PRIV(vis), vis)
+#define MANSYNC_cont(vis)   MANSYNC_DECL_CONT(AA_PRIV(vis), vis)
 
-typedef struct { 
+typedef struct {
 	aa_context *context;
-	aa_palette pal;
-	
-	_ggi_opmansync *opmansync;
+
+	MANSYNC_DATA;
 	void *aalock;
 
 	int fastrender;
