@@ -1,4 +1,4 @@
-/* $Id: demo.c,v 1.6 2002/05/19 15:42:43 skids Exp $
+/* $Id: demo.c,v 1.21 2005/07/30 11:58:39 cegger Exp $
 ******************************************************************************
 
    demo.c - the main LibGGI demo
@@ -30,41 +30,82 @@
 # define srandom	srand
 #endif
 
+/* Include the LibGGI declarations.
+ */
+#include <ggi/ggi.h>
+#include <ggi/gg.h>
+
 /* Include the necessary headers used for e.g. error-reporting.
  */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 
-/* Include the LibGGI declarations.
+/* A couple of routines that tries to put random bits in
+ * every position of 16 and 32 bit numbers.
  */
-#include <ggi/ggi.h>
+static int random_16_shift = 0;
+static int random_32_shift = 0;
+static inline uint16_t random16(void)
+{
+	uint16_t rnd = random();
+	int shift = 16 - random_16_shift;
+	while (shift < 16) {
+		rnd |= random() << shift;
+		shift += 16 - random_16_shift;
+	} 
+	return rnd;
+}
+static inline uint32_t random32(void)
+{
+	uint32_t rnd = random();
+	int shift = 32 - random_32_shift;
+	while(shift < 32) {
+		rnd |= random() << shift;
+		shift += 32 - random_32_shift;
+	}
+	return rnd;
+}
+/* If RAND_MAX is less than 2^16-1, shift the value
+ * so that we get the brightest possible colors.
+ */
+#define RANDOM_COLOR(col) \
+	col.r = random() << random_16_shift; \
+	col.g = random() << random_16_shift; \
+	col.b = random() << random_16_shift;
 
 /* We are running on a single primary visual which is made accessible
  * globally so subroutines like "waitabit" can access it.
  * See its allocation in main() for details.
  */
-ggi_visual_t vis;
+static ggi_visual_t vis;
+
+
+/* noninteractive ? default no, interactive */
+
+static int noninteractive = 0;
 
 
 /* In case we were called with wrong parameters, give an explanation.
  */
-void usage(const char *prog)
+static void usage(const char *prog)
 {
 	fprintf(stderr, "Usage:\n\n"
-		        "%s [--clip] [--target <target>] "
+			"%s [--clip] [--noninteractive] [--target <target>] "
 			"<xsize>x<ysize>[#<virtx>x<virty>]"
 			"['['<bpp>']']\n\n"
-		       "Example: %s 320x200[8]\n"
+			"Example: %s 320x200[8]\n"
 		"For textmode, try %s 80x25T\n",prog,prog,prog);
 	exit(1);
 }
 
 /* Wait for a keypress. Shut down everything, if "q" is pressed.
  */
-void waitabit(void)
+static void waitabit(void)
 {
 	int key;
 
@@ -72,26 +113,34 @@ void waitabit(void)
 	 */
 	ggiFlush(vis);
 
-#if 0	/* Activate this to test waiting for max times. 
+	/* Activate this to test waiting for max times. 
 	 * It will cause the wait to not block indefinitely but only for
 	 * 5 seconds.
-         */
+	 */
+	if (noninteractive)
 	{
+#if 1
 		struct timeval tv={5,0};
 		key=ggiEventPoll(vis,emKeyPress,&tv);
 		if (! (key & emKeyPress)) return;
 		key = ggiGetc(vis);
-	}
-
 #else
-	/* ggiGetc will blocking-wait for a keypress. 
-	 * we ignore modifier keys.
-	 */
-	do {
-		key = ggiGetc(vis);
-
-	} while ((key == GIIK_VOID) || (GII_KTYP(key) == GII_KT_MOD));
+		gii_event ev;
+		ggUSleep(5000000);
+		if (! (ggiEventsQueued(vis, emKeyPress | emKeyRepeat)))
+			return;
+		ggiEventRead(vis, &ev, emKeyPress | emKeyRepeat);
+		key = ev.key.sym;
 #endif
+	} else {
+		/* ggiGetc will blocking-wait for a keypress.
+		 * we ignore modifier keys.
+		 */
+		do {
+			key = ggiGetc(vis);
+
+		} while ((key == GIIK_VOID) || (GII_KTYP(key) == GII_KT_MOD));
+	}
 
 	if ((key == 'q') || (key == 'Q'))	/* Q pressed */
 	{	
@@ -103,12 +152,12 @@ void waitabit(void)
 
 /* Pixel value for white and black. See main() on how to get at it.
  */
-ggi_pixel white;
-ggi_pixel black;
+static ggi_pixel white;
+static ggi_pixel black;
 
 /* Print the name of the current test in the top left corner.
  */
-void TestName(const char *name)
+static void TestName(const char *name)
 {
 	int left, right, top, bottom;
 	int ch_width=8, ch_height=8;
@@ -155,12 +204,12 @@ void TestName(const char *name)
 */
 static struct timeval test_start_time;
 
-void TestStart(void)
+static void TestStart(void)
 {
 	ggCurTime(&test_start_time);
 }
 
-int TestTime(void)	/* result is in seconds */
+static int TestTime(void)	/* result is in seconds */
 {
 	int seconds;
 	int micros;
@@ -248,6 +297,7 @@ int main(int argc, char **argv)
 	 * Hope this is enough ... 
 	 */
 	char put_buf[64*1024];
+	char get_buf[64*1024];
 
 	/* This is a struct containing visual and virtual screen size as
 	 * well as the bpp/textmode information 
@@ -262,6 +312,22 @@ int main(int argc, char **argv)
 		{GGI_AUTO,GGI_AUTO}     /* Font size */
 	};
 
+
+	/* Calculate how many bits the random numbers needs to be
+	 * shifted in order to affect the most significant bit of the
+	 * colors.
+	 */
+	uint32_t tmp_rand = RAND_MAX;
+	while (tmp_rand < 0x8000U) {
+		++random_16_shift;
+		++random_32_shift;
+		tmp_rand <<= 1;
+	}
+	while (tmp_rand < 0x80000000U) {
+		++random_32_shift;
+		tmp_rand <<= 1;
+	}
+
 	/* Get the arguments from the command line. 
 	 * Set defaults for optional arguments.
 	 */
@@ -284,21 +350,31 @@ int main(int argc, char **argv)
 		/* Enable clipping */
 		doclip=1; argv++; argc--;
 	}
-	
+
+	if ((argc > 0) &&
+	    ((strcmp(*argv, "-n") == 0) ||
+	     (strcmp(*argv, "--noninteractive") == 0))) {
+
+		/* toggle noninteractive */
+		noninteractive = !noninteractive; argv++; argc--;
+	}
+	printf("running tests %sinteractively.\n",
+			noninteractive?"non-":"");
+
 	if ((argc > 1) && 
 	    ((strcmp(*argv, "-t") == 0) ||
 	     (strcmp(*argv, "--target") == 0))) {
 
 		target_name=argv[1]; argv+=2; argc-=2;
 	}
-	
+
 	if (argc > 1) {
 		usage(prog);	/* This is not an allowed call format. */
 		return 1;	/* Tell the user how to call and fail. */
 	}
-	
+
 	/* Set up the random number generator. */
-	srandom(time(NULL));
+	srandom((unsigned)time(NULL));
 
 	/* Initialize the GGI library. This must be called before any other 
 	 * GGI function. Otherwise behaviour is undefined.
@@ -405,8 +481,9 @@ int main(int argc, char **argv)
 	sx_5 = sx/5; sx_12 = sx/12;
 	sy_5 = sy/5; sy_12 = sy/12;
 
-	printf("Visual size is %d mm x %d mm (0 means unknown)\n", mode.size.x, mode.size.y);
-	
+	printf("Visual size is %d mm x %d mm (0 means unknown)\n",
+			mode.size.x, mode.size.y);
+
 	/* Set a colorful palette for the tests.
 	   Please note that GGI always uses 16 bit color components,
 	   so stretch the values accordingly when porting from DOS 
@@ -429,11 +506,11 @@ int main(int argc, char **argv)
 	map[0].b=0xFFFF;
 	
 	white=ggiMapColor(vis, &map[0]);
-	printf("white=%d\n", white);
+	printf("white=%u\n", white);
 
 	map[0].r= map[0].g= map[0].b= 0x0;
 	black=ggiMapColor(vis, &map[0]);
-	printf("black=%d\n", black);
+	printf("black=%u\n", black);
 
 	/* Set the drawing color to black and clear the screen.
 	   (The screen is cleared to black at setmode time, but the
@@ -446,14 +523,17 @@ int main(int argc, char **argv)
 	 */
 	ggiSetGCForeground(vis, white);
 	ggiSetGCBackground(vis, black);
-	ggiPuts(vis,0,0,"Press any key to begin tests...");
+	if (noninteractive)
+		ggiPuts(vis,0,0,"Beginning tests...");
+	else
+		ggiPuts(vis,0,0,"Press any key to begin tests...");
 	ggiFlush(vis);
 
 	/* Wait for any keypress. 
 	 * This is a blocking call which returns a 16 bit wide unicode
 	 * character
 	 */
-	ggiGetc(vis);
+	waitabit();
 
 	if (doclip) {
 		ggiDrawHLine(vis, 0, sy_5-1,  sx);
@@ -480,16 +560,12 @@ int main(int argc, char **argv)
 		ggi_color col;
 
 		/* Set the foreground color to some random value */
-		col.r = random();
-		col.g = random();
-		col.b = random();
+		RANDOM_COLOR(col);
 
 		ggiSetGCForeground(vis, ggiMapColor(vis, &col));
 
 		/* Set the background color to some random value */
-		col.r = random();
-		col.g = random();
-		col.b = random();
+		RANDOM_COLOR(col);
 
 		ggiSetGCBackground(vis, ggiMapColor(vis, &col));
 
@@ -506,6 +582,19 @@ int main(int argc, char **argv)
 		if (ggiKbhit(vis)) break;
 	}
 	printf("Puts(): %d strings\n", i);
+	ggiSetGCBackground(vis, black);	
+	ggiSetGCForeground(vis, white);	
+	i = 25;
+	ggiPutc(vis, i, i, (signed)'P'); i+= 10;
+	ggiPutc(vis, i, i, (signed)'u'); i+= 10;
+	ggiPutc(vis, i, i, (signed)'t'); i+= 10;
+	ggiPutc(vis, i, i, (signed)'c'); i+= 10;
+	ggiPutc(vis, i, i, (signed)' '); i+= 10;
+	ggiPutc(vis, i, i, (signed)'T'); i+= 10;
+	ggiPutc(vis, i, i, (signed)'e'); i+= 10;
+	ggiPutc(vis, i, i, (signed)'s'); i+= 10;
+	ggiPutc(vis, i, i, (signed)'t'); i+= 10;
+
 
 	/* Check the colormapping code. Draw four horizontal bars in red/green
 	 * blue and grey.
@@ -518,7 +607,7 @@ int main(int argc, char **argv)
 		/* Make i (intensity) so it varies from 0-0xFFFF over
 		 * the visible length
 		 */
-		int i = 0xFFFF*x/(sx-1);
+		i = 0xFFFF*x/(sx-1);
 		
 		/* Now first make a color descriptor for the red bar. It
 		 * will holf the R/G/B triplet x/0/0 describing the
@@ -527,7 +616,7 @@ int main(int argc, char **argv)
 		map[0].r=i; map[0].g=0; map[0].b=0;
 
 		/* ggiMapColor will return the color value that is
-		 * closest to the requested color. We set this a ctive
+		 * closest to the requested color. We set this as active
 		 * drawing color.
 		 */
 		ggiSetGCForeground(vis,ggiMapColor(vis, &map[0]));
@@ -553,7 +642,6 @@ int main(int argc, char **argv)
 		ggiDrawVLine(vis, x, 7*sy_12, sy_12);
 	}
 
-
 	/* Now the memvisual and crossblitting code.
 	 * This is an advanced topic which you should ignore if you are
 	 * a beginner.
@@ -585,14 +673,10 @@ int main(int argc, char **argv)
 	for (i=0; i < 50; i++) {
 		ggi_color col;
 
-		col.r = random();
-		col.g = random();
-		col.b = random();
+		RANDOM_COLOR(col);
 		ggiSetGCForeground(memvis, ggiMapColor(memvis, &col));
 
-		col.r = random();
-		col.g = random();
-		col.b = random();
+		RANDOM_COLOR(col);
 		ggiSetGCBackground(memvis, ggiMapColor(memvis, &col));
 
 		x = random() % 150;
@@ -660,19 +744,24 @@ int main(int argc, char **argv)
 
 	/* SetOrigin Y test - doesn't need to work 
 	 */
-	dy=0;
 	if (mode.visible.y < mode.virt.y) {
 
 		TestName("SetOrigin Y");
 		waitabit();
 
-		for (dy=0; dy < mode.virt.y-mode.visible.y; dy++) 
+		dx=dy=0;
+		TestStart();
+		for (; dy < mode.virt.y-mode.visible.y; dy++)
 		{
 			ggiSetOrigin(vis,0,dy);
 			ggUSleep(2000);
-			if (ggiKbhit(vis)) break;
+			if (ggiKbhit(vis) || (dx = TestTime()) > 9) break;
 		}
+		dx = TestTime();
 		ggiSetOrigin(vis,0,0);
+		printf("SetOrigin Y: offset %i took %d seconds"
+				" -- %.3f steps per second\n", dy, dx,
+				(double)((dx>0)?(dy/dx):dy));
 	}
 
 	/* SetOrigin X test - doesn't need to work.
@@ -683,19 +772,25 @@ int main(int argc, char **argv)
 		TestName("SetOrigin X");
 		waitabit();
 
-		for (dx=0; dx < mode.virt.x-mode.visible.x; dx++) 
+		dx=dy=0;
+		TestStart();
+		for (; dx < mode.virt.x-mode.visible.x; dx++)
 		{
 			ggiSetOrigin(vis,dx,0);
 			ggUSleep(2000);
-			if (ggiKbhit(vis)) break;
+			if (ggiKbhit(vis) || (dy = TestTime()) > 9) break;
 		}
+		dy = TestTime();
 		ggiSetOrigin(vis,0,0);
+		printf("SetOrigin X: offset %i took %d seconds"
+				" -- %.3f steps per second\n", dx, dy,
+				(double)((dy>0)?(dx/dy):dx));
 	}
 
 	/* Hline tests.
 	 * Draw horizontal lines.
 	 */
-	TestName("HLine");
+	TestName("HLine Draw");
 	waitabit();
 
 	ggiSetGCForeground(vis, black);
@@ -719,11 +814,12 @@ int main(int argc, char **argv)
 		}
 		if (ggiKbhit(vis)) break;
 	}
+	printf("DrawHLine(): %d lines\n", c);
 
 	/* VLine Tests.
 	 * Draw vertical lines.
 	 */
-	TestName("VLine");
+	TestName("VLine Draw");
 	waitabit();
 
 	ggiSetGCForeground(vis, black);
@@ -740,13 +836,14 @@ int main(int argc, char **argv)
 			
 			ggiSetGCForeground(vis, ggiMapColor(vis, &col));
 
-			/* Draw vertical lines at x=vx-i-1, y=i, and with
+			/* Draw vertical lines at x=sx-i-1, y=i, and with
 			 * height=sy / 5.
 			 */
-			ggiDrawVLine(vis, vx-i-1, i, sy_5);
+			ggiDrawVLine(vis, sx-i-1, i, sy_5);
 		}
 		if (ggiKbhit(vis)) break;
 	}
+	printf("DrawVLine(): %d lines\n", c);
 
 	/* Put horizontal lines. "Putting" means blitting the contents
 	 * of a memory buffer.  
@@ -759,12 +856,59 @@ int main(int argc, char **argv)
 	TestName("Hline Put");
 	waitabit();
 
-	for(x=0; x<sizeof(put_buf); x++) put_buf[x]=x&0xff;
+	for(x = 0; x < (signed)sizeof(put_buf); x++) put_buf[x]=x&0xff;
 
-	for(y=10; y<vy; y++) {
+	for(y=15; y<vy; y++) {
 		ggiPutHLine(vis, 0, y, vx, put_buf);
 	}
 
+	/* Get horizontal lines. */
+	TestName("Hline Get");
+	memset(get_buf, 0, sizeof(get_buf)); /* clear get_buf */
+	x = 0; /* count wrong gets */
+
+	for(y=15; y<vy; y++) {
+		ggiGetHLine(vis, 0, y, vx, get_buf);
+		if (get_buf[y] != put_buf[y]) {
+			x += 1;
+#if 0
+			fprintf(stderr,"Error: Hline[%i]: put %x != get %x\n",
+					y, put_buf[y], get_buf[y]);
+#endif
+		}
+	}
+	/* this does not account for clipping */
+	if (!doclip && x)
+		fprintf(stderr, "ggiGetHline: %i errors.\n", x);
+
+	/* The same with vlines.
+	 */
+	TestName("Vline Put");
+	waitabit();
+
+	for(x=0; x<vx; x++) {
+		ggiPutVLine(vis, x, 15, vy - 15, put_buf);
+	}
+
+	/* Get vertical lines. */
+	TestName("Vline Get");
+	memset(get_buf, 0, sizeof(get_buf)); /* clear get_buf */
+	y = 0; /* count wrong gets */
+
+	for(x=0; x<vx; x++) {
+		ggiGetVLine(vis, x, 15, vy - 15, get_buf);
+		if (get_buf[x] != put_buf[x]) {
+			y += 1;
+#if 0
+			fprintf(stderr,"Error: Vline[%i]: put %x != get %x\n",
+					x, put_buf[x], get_buf[x]);
+#endif
+		}
+	}
+	/* this does not account for clipping */
+	if (!doclip && y)
+		fprintf(stderr, "ggiGetVline: %i errors.\n", y);
+
 	/* The same with boxes.
 	 */
 	TestName("PutBox");
@@ -795,9 +939,7 @@ int main(int argc, char **argv)
 
 	for (i=0; TestTime() < 3; i++) {
 		ggi_color col;
-		col.r = random();
-		col.g = random();
-		col.b = random();
+		RANDOM_COLOR(col);
 
 		x = random() % vx;
 		y = random() % vy;
@@ -808,6 +950,7 @@ int main(int argc, char **argv)
 		ggiDrawBox(vis, x, y, w, h);
 		if (ggiKbhit(vis)) break;
 	}
+	printf("Boxes(): %d boxes\n", i);
 
 	/* Linedrawing tests */
 	TestName("Lines");
@@ -820,9 +963,7 @@ int main(int argc, char **argv)
 	for (i=0; TestTime() < 3; i++) {
 		ggi_color col;
 
-		col.r = random();
-		col.g = random();
-		col.b = random();
+		RANDOM_COLOR(col);
 
 		x = random() % vx;
 		y = random() % vy;
@@ -833,8 +974,8 @@ int main(int argc, char **argv)
 		ggiDrawLine(vis, x, y, w, h);
 		if (ggiKbhit(vis)) break;
 	}
-	printf("DrawLine(): %d lines\n", i);
-		
+	printf("Lines(): %d lines\n", i);
+
 	/* CopyBox tests */
 
 	TestName("CopyBox");
@@ -894,7 +1035,7 @@ int main(int argc, char **argv)
 		map[i].b=0xffff*i/127;
 	}
 
-	/* Convert it to get/put buffer format, and the blit it to the
+	/* Convert it to get/put buffer format, and then blit it to the
 	 * screen.
 	 */
 	ggiPackColors(vis, put_buf, map, 128);
@@ -979,18 +1120,18 @@ int main(int argc, char **argv)
 		i = 0;
 		while ((TestTime() < 10) && (i < numplanes)) {
 			for (y = 0; (TestTime() < 15) && (y < vy); y++) {
-			  	uint8 *linestart;
-				linestart = (uint8 *)dbuf->write + 
-				  stride2 * i + stride * y;
+				uint8_t *linestart;
+				linestart = (uint8_t *)dbuf->write + 
+					stride2 * i + stride * y;
 				x = 0;
-				if (ggiKbhit(vis)) goto dbuf_end;
-				while (x < vx * GT_SIZE(type)/wordsize) {
+				if (ggiKbhit(vis)) goto dbuf_tidy;
+				while (x < vx * (signed)GT_SIZE(type)/wordsize) {
 					switch(wordsize) {
 					case 32:
-					  *((uint32 *)linestart+x) = random();
+					  *((uint32_t *)linestart+x) = random32();
 					  break;
 					case 16:
-					  *((uint16 *)linestart+x) = random();
+					  *((uint16_t *)linestart+x) = random16();
 					  break;
 					case 8:
 					  *(linestart+x) = random();
@@ -1000,26 +1141,26 @@ int main(int argc, char **argv)
 				}
 			}
 			i++;
-		}	
+		}
 
 		while (TestTime() < 10) {
-			uint8 *linestart;
+			uint8_t *linestart;
 
-			if (ggiKbhit(vis)) goto dbuf_end;
+			if (ggiKbhit(vis)) goto dbuf_tidy;
 
 			i = random() % numplanes;
 			y = random() % vy;
 			x = random() % (vx * GT_SIZE(type)/wordsize);
-			  	
-			linestart = (uint8 *)dbuf->write + 
+
+			linestart = (uint8_t *)dbuf->write + 
 			  stride2 * i + stride * y;
-			
+
 			switch(wordsize) {
 			case 32:
-				*((uint32 *)linestart+x) = random();
+				*((uint32_t *)linestart+x) = random32();
 				break;
 			case 16:
-				*((uint16 *)linestart+x) = random();
+				*((uint16_t *)linestart+x) = random16();
 				break;
 			case 8:
 				*(linestart+x) = random();
@@ -1027,15 +1168,55 @@ int main(int argc, char **argv)
 			}
 		}
 
+	dbuf_tidy:
+
+		ggiAddFlags(vis, GGIFLAG_TIDYBUF);
+		if (ggiGetFlags(vis) & GGIFLAG_TIDYBUF) {
+			TestName("Directbuffer (with GGIFLAG_TIDYBUF flag)");
+			waitabit();
+			TestStart();
+
+			i = 0;
+			dx = (100 < vx) ? 100 : vx;
+			dy = (100 < vy) ? 100 : vy;
+
+			c = 0;
+			while (TestTime() < 10) {
+				uint8_t *linestart;
+
+				if (ggiKbhit(vis)) goto dbuf_end;
+
+				i = random() % numplanes;
+				y = (vy - dy) / 2 + random() % dy;
+				x = (vx - dx) / 2 + random() % dx;
+				x *= GT_SIZE(type);
+				x /= wordsize;
+
+				linestart = (uint8_t *)dbuf->write + 
+				  stride2 * i + stride * y;
+
+				switch(wordsize) {
+				case 32:
+					*((uint32_t *)linestart+x) = random32();
+					break;
+				case 16:
+					*((uint16_t *)linestart+x) = random16();
+					break;
+				case 8:
+					*(linestart+x) = random();
+					break;
+				}
+
+				c++;
+				if (c > 100) {
+					ggiFlushRegion(vis, 
+						       (vx-dx)/2,(vy-dy)/2,
+						       dx,dy);
+					c = 0;
+				}
+			}
+		}	/* if */
 	dbuf_end:
-		/* If we were not in syncronous mode, we would
-		 * call ggiFlush here, because there is no guarantee
-		 * that the directbuffer is not a software back-buffer
-		 * which still needs to be sent to the display.
-		 * 
-		 * We release our directbuffer immediately once we are done,
-		 * even if we are going to use it again in a short time.
-		 */
 		ggiResourceRelease(dbuf->resource);
 
 	} while (0);
@@ -1055,6 +1236,7 @@ int main(int argc, char **argv)
 	 */
 	
 	ggiSetFlags(vis,GGIFLAG_ASYNC);
+	if (!(ggiGetFlags(vis) & GGIFLAG_ASYNC)) goto palette;
 	
 	TestName("Async mode");
 	waitabit();
@@ -1063,9 +1245,7 @@ int main(int argc, char **argv)
 	for (i=0; TestTime() < 3; i++) {
 		ggi_color col;
 
-		col.r = random();
-		col.g = random();
-		col.b = random();
+		RANDOM_COLOR(col);
 
 		x = random() % vx;
 		y = random() % vy;
@@ -1081,6 +1261,7 @@ int main(int argc, char **argv)
 		if (ggiKbhit(vis)) break;
 	}
 
+ palette:
 	/* Palette Test. We do some colorcycling here.
 	 */
 	if (GT_SCHEME(mode.graphtype) == GT_PALETTE) {
@@ -1163,7 +1344,7 @@ int main(int argc, char **argv)
 	 * undefined behaviour now. It is not recommended to needlessly
 	 * deinit-reinit LibGGI, but it's possible.
 	 */
-	ggiExit();	
+	ggiExit();
 
 	/* Terminate the program.
 	 */
